@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qlocale_p.h"
 #include "qlocale_tools_p.h"
@@ -46,6 +10,8 @@
 #include "qdatetime.h"
 #include "qdebug.h"
 
+#include "QtCore/private/qgregoriancalendar_p.h" // for yearSharingWeekDays()
+
 #ifdef Q_OS_WIN
 #   include <qt_windows.h>
 #   include <time.h>
@@ -53,6 +19,7 @@
 
 #if QT_CONFIG(cpp_winrt) && !defined(Q_CC_CLANG)
 #   include <winrt/base.h>
+#   include <QtCore/private/qfactorycacheregistration_p.h>
 // Workaround for Windows SDK bug.
 // See https://github.com/microsoft/Windows.UI.Composition-Win32-Samples/issues/47
 namespace winrt::impl
@@ -67,6 +34,8 @@ namespace winrt::impl
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 static QByteArray getWinLocaleName(LCID id = LOCALE_USER_DEFAULT);
 static QString winIso639LangName(LCID id = LOCALE_USER_DEFAULT);
 static QString winIso3116CtryName(LCID id = LOCALE_USER_DEFAULT);
@@ -149,6 +118,7 @@ private:
 
     SubstitutionType substitution();
     QString substituteDigits(QString &&string);
+    QString yearFix(int year, int fakeYear, QString &&formatted);
 
     static QString winToQtFormat(QStringView sys_fmt);
 
@@ -255,7 +225,7 @@ QString QSystemLocalePrivate::substituteDigits(QString &&string)
             break;
         Q_ASSERT(z > '9');
         ushort *const qch = reinterpret_cast<ushort *>(string.data());
-        for (int i = 0, stop = string.size(); i < stop; ++i) {
+        for (qsizetype i = 0, stop = string.size(); i < stop; ++i) {
             ushort &ch = qch[i];
             if (ch >= '0' && ch <= '9')
                 ch = unicodeForDigit(ch - '0', z);
@@ -348,7 +318,7 @@ QVariant QSystemLocalePrivate::timeFormat(QLocale::FormatType type)
 
 QVariant QSystemLocalePrivate::dateTimeFormat(QLocale::FormatType type)
 {
-    return QString(dateFormat(type).toString() + QLatin1Char(' ') + timeFormat(type).toString());
+    return QString(dateFormat(type).toString() + u' ' + timeFormat(type).toString());
 }
 
 QVariant QSystemLocalePrivate::dayName(int day, QLocale::FormatType type)
@@ -395,13 +365,12 @@ QVariant QSystemLocalePrivate::standaloneMonthName(int month, QLocale::FormatTyp
             LOCALE_SMONTHNAME7, LOCALE_SMONTHNAME8, LOCALE_SMONTHNAME9,
             LOCALE_SMONTHNAME10, LOCALE_SMONTHNAME11, LOCALE_SMONTHNAME12 };
 
-    month -= 1;
-    if (month < 0 || month > 11)
+    if (month < 1 || month > 12)
         return {};
 
-    LCTYPE lctype = (type == QLocale::ShortFormat || type == QLocale::NarrowFormat)
-            ? short_month_map[month] : long_month_map[month];
-    return getLocaleInfo(lctype);
+    // Month is Jan = 1, ... Dec = 12; adjust by 1 to match array indexing from 0:
+    return getLocaleInfo(
+        (type == QLocale::LongFormat ? long_month_map : short_month_map)[month - 1]);
 }
 
 QVariant QSystemLocalePrivate::monthName(int month, QLocale::FormatType type)
@@ -428,20 +397,81 @@ QVariant QSystemLocalePrivate::monthName(int month, QLocale::FormatType type)
     return {};
 }
 
+static QString fourDigitYear(int year)
+{
+    // Return year formatted as an (at least) four digit number:
+    return QStringLiteral("%1").arg(year, 4, 10, QChar(u'0'));
+}
+
+QString QSystemLocalePrivate::yearFix(int year, int fakeYear, QString &&formatted)
+{
+    // Replace our ersatz fakeYear (that MS formats faithfully) with the correct
+    // form of year.  We know the two-digit short form of fakeYear can not be
+    // mistaken for the month or day-of-month in the formatted date.
+    Q_ASSERT(fakeYear >= 1970 && fakeYear <= 2400);
+    const bool matchTwo = year >= 0 && year % 100 == fakeYear % 100;
+    auto yearUsed = fourDigitYear(fakeYear);
+    QString sign(year < 0 ? 1 : 0, u'-');
+    auto trueYear = fourDigitYear(year < 0 ? -year : year);
+    if (formatted.contains(yearUsed))
+        return std::move(formatted).replace(yearUsed, sign + trueYear);
+
+    auto tail = QStringView{yearUsed}.last(2);
+    Q_ASSERT(!matchTwo || tail == QString(sign + trueYear.last(2)));
+    if (formatted.contains(tail)) {
+        if (matchTwo)
+            return std::move(formatted);
+        return std::move(formatted).replace(tail.toString(), sign + trueYear.last(2));
+    }
+
+    // Localized digits, perhaps ?
+    // First call to substituteDigits() ensures zero is initialized:
+    trueYear = substituteDigits(std::move(trueYear));
+    if (zero != u'0') {
+        yearUsed = substituteDigits(std::move(yearUsed));
+        if (year < 0)
+            sign = negativeSign().toString();
+
+        if (formatted.contains(yearUsed))
+            return std::move(formatted).replace(yearUsed, sign + trueYear);
+
+        const qsizetype twoDigits = 2 * zero.size();
+        tail = QStringView{yearUsed}.last(twoDigits);
+        if (formatted.contains(tail)) {
+            if (matchTwo)
+                return std::move(formatted);
+            return std::move(formatted).replace(tail.toString(), sign + trueYear.last(twoDigits));
+        }
+    }
+    qWarning("Failed to fix up year in formatted date-string using %d for %d", fakeYear, year);
+    return std::move(formatted);
+}
+
 QVariant QSystemLocalePrivate::toString(QDate date, QLocale::FormatType type)
 {
     SYSTEMTIME st = {};
-    st.wYear = date.year();
+    const int year = date.year();
+    // st.wYear is unsigned; and GetDateFormat() is documented to not handle
+    // dates before 1601.
+    const bool fixup = year < 1601;
+    st.wYear = fixup ? QGregorianCalendar::yearSharingWeekDays(date) : year;
     st.wMonth = date.month();
     st.wDay = date.day();
 
+    Q_ASSERT(!fixup || st.wYear % 100 != st.wMonth);
+    Q_ASSERT(!fixup || st.wYear % 100 != st.wDay);
+    // i.e. yearFix() can trust a match of its fakeYear's last two digits to not
+    // be the month or day part of the formatted date.
+
     DWORD flags = (type == QLocale::LongFormat ? DATE_LONGDATE : DATE_SHORTDATE);
     wchar_t buf[255];
     if (getDateFormat(flags, &st, NULL, buf, 255)) {
-        QString format = QString::fromWCharArray(buf);
+        QString text = QString::fromWCharArray(buf);
+        if (fixup)
+            text = yearFix(year, st.wYear, std::move(text));
         if (substitution() == SAlways)
-            format = substituteDigits(std::move(format));
-        return format;
+            text = substituteDigits(std::move(text));
+        return text;
     }
     return {};
 }
@@ -461,17 +491,18 @@ QVariant QSystemLocalePrivate::toString(QTime time, QLocale::FormatType type)
 
     wchar_t buf[255];
     if (getTimeFormat(flags, &st, NULL, buf, 255)) {
-        QString format = QString::fromWCharArray(buf);
+        QString text = QString::fromWCharArray(buf);
         if (substitution() == SAlways)
-            format = substituteDigits(std::move(format));
-        return format;
+            text = substituteDigits(std::move(text));
+        return text;
     }
     return {};
 }
 
 QVariant QSystemLocalePrivate::toString(const QDateTime &dt, QLocale::FormatType type)
 {
-    return QString(toString(dt.date(), type).toString() + QLatin1Char(' ') + toString(dt.time(), type).toString());
+    return QString(toString(dt.date(), type).toString() + u' '
+                   + toString(dt.time(), type).toString());
 }
 
 QVariant QSystemLocalePrivate::measurementSystem()
@@ -479,10 +510,8 @@ QVariant QSystemLocalePrivate::measurementSystem()
     wchar_t output[2];
 
     if (getLocaleInfo(LOCALE_IMEASURE, output, 2)) {
-        QString iMeasure = QString::fromWCharArray(output);
-        if (iMeasure == QLatin1String("1")) {
+        if (output[0] == L'1' && !output[1])
             return QLocale::ImperialSystem;
-        }
     }
 
     return QLocale::MetricSystem;
@@ -604,7 +633,7 @@ QVariant QSystemLocalePrivate::toCurrencyString(const QSystemLocale::CurrencyToS
         // int(32) == "12,34,56,789.00" == string("3;2;0")
         // int(320)== "1234,56,789.00"  == string("3;2")
         QString groupingStr = getLocaleInfo(LOCALE_SMONGROUPING).toString();
-        format.Grouping = groupingStr.remove(QLatin1Char(';')).toInt();
+        format.Grouping = groupingStr.remove(u';').toInt();
         if (format.Grouping % 10 == 0) // magic
             format.Grouping /= 10;
         else
@@ -634,9 +663,13 @@ QVariant QSystemLocalePrivate::uiLanguages()
 #if QT_CONFIG(cpp_winrt) && !defined(Q_CC_CLANG)
     using namespace winrt;
     using namespace Windows::System::UserProfile;
-    auto languages = GlobalizationPreferences::Languages();
-    for (const auto &lang : languages)
-        result << QString::fromStdString(winrt::to_string(lang));
+    QT_TRY {
+        auto languages = GlobalizationPreferences::Languages();
+        for (const auto &lang : languages)
+            result << QString::fromStdString(winrt::to_string(lang));
+    } QT_CATCH(...) {
+        // pass, just fall back to WIN32 API implementation
+    }
     if (!result.isEmpty())
         return result; // else just fall back to WIN32 API implementation
 #endif // QT_CONFIG(cpp_winrt) && !defined(Q_CC_CLANG)
@@ -688,20 +721,20 @@ void QSystemLocalePrivate::update()
 QString QSystemLocalePrivate::winToQtFormat(QStringView sys_fmt)
 {
     QString result;
-    int i = 0;
+    qsizetype i = 0;
 
     while (i < sys_fmt.size()) {
-        if (sys_fmt.at(i).unicode() == QLatin1Char('\'')) {
+        if (sys_fmt.at(i).unicode() == u'\'') {
             QString text = qt_readEscapedFormatString(sys_fmt, &i);
-            if (text == QLatin1String("'"))
-                result += QLatin1String("''");
+            if (text == "'"_L1)
+                result += "''"_L1;
             else
-                result += QLatin1Char('\'') + text + QLatin1Char('\'');
+                result += u'\'' + text + u'\'';
             continue;
         }
 
         QChar c = sys_fmt.at(i);
-        int repeat = qt_repeatCount(sys_fmt.mid(i));
+        qsizetype repeat = qt_repeatCount(sys_fmt.mid(i));
 
         switch (c.unicode()) {
             // Date
@@ -712,13 +745,13 @@ QString QSystemLocalePrivate::winToQtFormat(QStringView sys_fmt)
                     repeat = 2;
                 switch (repeat) {
                     case 1:
-                        result += QLatin1String("yy"); // "y" unsupported by Qt, use "yy"
+                        result += "yy"_L1; // "y" unsupported by Qt, use "yy"
                         break;
                     case 5:
-                        result += QLatin1String("yyyy"); // "yyyyy" same as "yyyy" on Windows
+                        result += "yyyy"_L1; // "yyyyy" same as "yyyy" on Windows
                         break;
                     default:
-                        result += QString(repeat, QLatin1Char('y'));
+                        result += QString(repeat, u'y');
                         break;
                 }
                 break;
@@ -729,14 +762,14 @@ QString QSystemLocalePrivate::winToQtFormat(QStringView sys_fmt)
                     case 2:
                         break; // no equivalent of "gg" in Qt
                     default:
-                        result += QLatin1Char('g');
+                        result += u'g';
                         break;
                 }
                 break;
             case 't':
                 if (repeat > 2)
                     repeat = 2;
-                result += QLatin1String("AP"); // "t" unsupported, use "AP"
+                result += "AP"_L1; // "t" unsupported, use "AP"
                 break;
             default:
                 result += QString(repeat, c);
@@ -1042,14 +1075,12 @@ static QString winIso639LangName(LCID id)
         lang_code = QString::fromWCharArray(out);
 
     if (!lang_code.isEmpty()) {
-        const char *endptr;
-        bool ok;
-        QByteArray latin1_lang_code = std::move(lang_code).toLatin1();
-        int i = qstrtoull(latin1_lang_code, &endptr, 16, &ok);
-        if (ok && *endptr == '\0') {
+        const QByteArray latin1 = std::move(lang_code).toLatin1();
+        const auto [i, endptr] = qstrntoull(latin1.data(), latin1.size(), 16);
+        if (endptr && *endptr == '\0') {
             switch (i) {
                 case 0x814:
-                    result = QLatin1String("nn"); // Nynorsk
+                    result = u"nn"_s; // Nynorsk
                     break;
                 default:
                     break;
@@ -1086,8 +1117,8 @@ static QByteArray getWinLocaleName(LCID id)
         result = langEnvVar;
         if (result == "C"
             || (!result.isEmpty() && qt_splitLocaleName(QString::fromLocal8Bit(result)))) {
-            bool ok = false; // See if we have a Windows locale code instead of a locale name:
-            long id = qstrtoll(result.data(), 0, 0, &ok);
+            // See if we have a Windows locale code instead of a locale name:
+            auto [id, ok] = qstrntoll(result.data(), result.size(), 0);
             if (!ok || id == 0 || id < INT_MIN || id > INT_MAX) // Assume real locale name
                 return result;
             return winLangCodeToIsoName(int(id));
@@ -1099,7 +1130,7 @@ static QByteArray getWinLocaleName(LCID id)
     QString resultusage = winIso639LangName(id);
     QString country = winIso3116CtryName(id);
     if (!country.isEmpty())
-        resultusage += QLatin1Char('_') + country;
+        resultusage += u'_' + country;
 
     return std::move(resultusage).toLatin1();
 }
