@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qthreadpool.h"
 #include "qthreadpool_p.h"
@@ -43,9 +7,12 @@
 #include "qcoreapplication.h"
 
 #include <algorithm>
+#include <memory>
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 /*
     QThread wrapper, provides synchronization against a ThreadPool
 */
@@ -221,7 +188,7 @@ inline bool comparePriority(int priority, const QueuePage *p)
 void QThreadPoolPrivate::enqueueTask(QRunnable *runnable, int priority)
 {
     Q_ASSERT(runnable != nullptr);
-    for (QueuePage *page : qAsConst(queue)) {
+    for (QueuePage *page : std::as_const(queue)) {
         if (page->priority() == priority && !page->isFull()) {
             page->push(runnable);
             return;
@@ -233,9 +200,9 @@ void QThreadPoolPrivate::enqueueTask(QRunnable *runnable, int priority)
 
 int QThreadPoolPrivate::activeThreadCount() const
 {
-    return (allThreads.count()
-            - expiredThreads.count()
-            - waitingThreads.count()
+    return (allThreads.size()
+            - expiredThreads.size()
+            - waitingThreads.size()
             + reservedThreads);
 }
 
@@ -274,16 +241,16 @@ bool QThreadPoolPrivate::tooManyThreadsActive() const
 void QThreadPoolPrivate::startThread(QRunnable *runnable)
 {
     Q_ASSERT(runnable != nullptr);
-    QScopedPointer<QThreadPoolThread> thread(new QThreadPoolThread(this));
+    auto thread = std::make_unique<QThreadPoolThread>(this);
     if (objectName.isEmpty())
-        objectName = QLatin1String("Thread (pooled)");
+        objectName = u"Thread (pooled)"_s;
     thread->setObjectName(objectName);
-    Q_ASSERT(!allThreads.contains(thread.data())); // if this assert hits, we have an ABA problem (deleted threads don't get removed here)
-    allThreads.insert(thread.data());
+    Q_ASSERT(!allThreads.contains(thread.get())); // if this assert hits, we have an ABA problem (deleted threads don't get removed here)
+    allThreads.insert(thread.get());
     ++activeThreads;
 
     thread->runnable = runnable;
-    thread.take()->start(threadPriority);
+    thread.release()->start(threadPriority);
 }
 
 /*!
@@ -302,7 +269,7 @@ void QThreadPoolPrivate::reset()
 
     mutex.unlock();
 
-    for (QThreadPoolThread *thread : qAsConst(allThreadsCopy)) {
+    for (QThreadPoolThread *thread : std::as_const(allThreadsCopy)) {
         if (thread->isRunning()) {
             thread->runnableReady.wakeAll();
             thread->wait();
@@ -381,7 +348,7 @@ bool QThreadPool::tryTake(QRunnable *runnable)
         return false;
 
     QMutexLocker locker(&d->mutex);
-    for (QueuePage *page : qAsConst(d->queue)) {
+    for (QueuePage *page : std::as_const(d->queue)) {
         if (page->tryTake(runnable)) {
             if (page->isFinished()) {
                 d->queue.removeOne(page);
@@ -498,8 +465,8 @@ QThreadPool::~QThreadPool()
 */
 QThreadPool *QThreadPool::globalInstance()
 {
-    static QPointer<QThreadPool> theInstance;
-    static QBasicMutex theMutex;
+    Q_CONSTINIT static QPointer<QThreadPool> theInstance;
+    Q_CONSTINIT static QBasicMutex theMutex;
 
     const QMutexLocker locker(&theMutex);
     if (theInstance.isNull() && !QCoreApplication::closingDown())
@@ -773,6 +740,57 @@ void QThreadPool::releaseThread()
     d->tryToStartMoreThreads();
 }
 
+/*!
+    Releases a thread previously reserved with reserveThread() and uses it
+    to run \a runnable.
+
+    Note that the thread pool takes ownership of the \a runnable if
+    \l{QRunnable::autoDelete()}{runnable->autoDelete()} returns \c true,
+    and the \a runnable will be deleted automatically by the thread
+    pool after the \l{QRunnable::run()}{runnable->run()} returns. If
+    \l{QRunnable::autoDelete()}{runnable->autoDelete()} returns \c false,
+    ownership of \a runnable remains with the caller. Note that
+    changing the auto-deletion on \a runnable after calling this
+    functions results in undefined behavior.
+
+    \note Calling this when no threads are reserved results in
+    undefined behavior.
+
+    \since 6.3
+    \sa reserveThread(), start()
+*/
+void QThreadPool::startOnReservedThread(QRunnable *runnable)
+{
+    if (!runnable)
+        return releaseThread();
+
+    Q_D(QThreadPool);
+    QMutexLocker locker(&d->mutex);
+    Q_ASSERT(d->reservedThreads > 0);
+    --d->reservedThreads;
+
+    if (!d->tryStart(runnable)) {
+        // This can only happen if we reserved max threads,
+        // and something took the one minimum thread.
+        d->enqueueTask(runnable, INT_MAX);
+    }
+}
+
+/*!
+    \overload
+    \since 6.3
+
+    Releases a thread previously reserved with reserveThread() and uses it
+    to run \a functionToRun.
+*/
+void QThreadPool::startOnReservedThread(std::function<void()> functionToRun)
+{
+    if (!functionToRun)
+        return releaseThread();
+
+    startOnReservedThread(QRunnable::create(std::move(functionToRun)));
+}
+
 /*!
     Waits up to \a msecs milliseconds for all threads to exit and removes all
     threads from the thread pool. Returns \c true if all threads were removed;
