@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qpalette.h"
 #include "qguiapplication.h"
@@ -52,7 +16,7 @@ static int qt_palette_count = 1;
 
 static constexpr QPalette::ResolveMask colorRoleOffset(QPalette::ColorGroup colorGroup)
 {
-    return QPalette::NColorRoles * colorGroup;
+    return qToUnderlying(QPalette::NColorRoles) * qToUnderlying(colorGroup);
 }
 
 static constexpr QPalette::ResolveMask bitPosition(QPalette::ColorGroup colorGroup,
@@ -69,12 +33,37 @@ static_assert(bitPosition(QPalette::ColorGroup(QPalette::NColorGroups - 1),
 class QPalettePrivate
 {
 public:
-    QPalettePrivate() : ref(1), ser_no(qt_palette_count++), detach_no(0) { }
+    class Data : public QSharedData {
+    public:
+        // Every instance of Data has to have a unique serial number, even
+        // if it gets created by copying another - we wouldn't create a copy
+        // in the first place if the serial number should be the same!
+        Data(const Data &other)
+            : QSharedData(other)
+        {
+            for (int grp = 0; grp < int(QPalette::NColorGroups); grp++) {
+                for (int role = 0; role < int(QPalette::NColorRoles); role++)
+                    br[grp][role] = other.br[grp][role];
+            }
+        }
+        Data() = default;
+
+        QBrush br[QPalette::NColorGroups][QPalette::NColorRoles];
+        const int ser_no = qt_palette_count++;
+    };
+
+    QPalettePrivate(const QExplicitlySharedDataPointer<Data> &data)
+        : ref(1), data(data)
+    { }
+    QPalettePrivate()
+        : QPalettePrivate(QExplicitlySharedDataPointer<Data>(new Data))
+    { }
+
     QAtomicInt ref;
-    QBrush br[QPalette::NColorGroups][QPalette::NColorRoles];
     QPalette::ResolveMask resolveMask = {0};
-    int ser_no;
-    int detach_no;
+    static inline int qt_palette_private_count = 0;
+    int detach_no = ++qt_palette_private_count;
+    QExplicitlySharedDataPointer<Data> data;
 };
 
 static QColor qt_mix_colors(QColor a, QColor b)
@@ -83,6 +72,34 @@ static QColor qt_mix_colors(QColor a, QColor b)
                   (a.blue() + b.blue()) / 2, (a.alpha() + b.alpha()) / 2);
 }
 
+/*!
+    \internal
+
+    Derive undefined \l PlaceholderText colors from \l Text colors.
+    Unless already set, PlaceholderText colors will be derived from their Text pendents.
+    Colors of existing PlaceHolderText brushes will not be replaced.
+
+    \a alpha represents the dim factor as a percentage. By default, a PlaceHolderText color
+    becomes a 50% more transparent version of the corresponding Text color.
+*/
+static void qt_placeholder_from_text(QPalette &pal, int alpha = 50)
+{
+    if (alpha < 0 or alpha > 100)
+        return;
+
+    for (int cg = 0; cg < int(QPalette::NColorGroups); ++cg) {
+        const QPalette::ColorGroup group = QPalette::ColorGroup(cg);
+
+        // skip if the brush has been set already
+        if (!pal.isBrushSet(group, QPalette::PlaceholderText)) {
+            QColor c = pal.color(group, QPalette::Text);
+            const int a = (c.alpha() * alpha) / 100;
+            c.setAlpha(a);
+            pal.setColor(group, QPalette::PlaceholderText, c);
+        }
+    }
+}
+
 static void qt_palette_from_color(QPalette &pal, const QColor &button)
 {
     int h, s, v;
@@ -105,6 +122,8 @@ static void qt_palette_from_color(QPalette &pal, const QColor &button)
     pal.setColorGroup(QPalette::Disabled, buttonBrushDark, buttonBrush, buttonBrushLight150,
                       buttonBrushDark, buttonBrushDark150, buttonBrushDark,
                       whiteBrush, buttonBrush, buttonBrush);
+
+    qt_placeholder_from_text(pal);
 }
 
 /*!
@@ -592,6 +611,8 @@ QPalette::QPalette(const QBrush &windowText, const QBrush &button,
     init();
     setColorGroup(All, windowText, button, light, dark, mid, text, bright_text,
                   base, window);
+
+    qt_placeholder_from_text(*this);
 }
 
 
@@ -647,6 +668,8 @@ QPalette::QPalette(const QColor &button, const QColor &window)
     setColorGroup(Disabled, disabledForeground, buttonBrush, buttonBrushLight150,
                   buttonBrushDark, buttonBrushDark150, disabledForeground,
                   whiteBrush, baseBrush, windowBrush);
+
+    qt_placeholder_from_text(*this);
 }
 
 /*!
@@ -746,7 +769,7 @@ const QBrush &QPalette::brush(ColorGroup gr, ColorRole cr) const
             gr = Active;
         }
     }
-    return d->br[gr][cr];
+    return d->data->br[gr][cr];
 }
 
 /*!
@@ -784,11 +807,18 @@ void QPalette::setBrush(ColorGroup cg, ColorRole cr, const QBrush &b)
         cg = Active;
     }
 
-    if (d->br[cg][cr] != b) {
+    const auto newResolveMask = d->resolveMask | ResolveMask(1) << bitPosition(cg, cr);
+    const auto valueChanged = d->data->br[cg][cr] != b;
+
+    if (valueChanged) {
+        detach();
+        d->data.detach();
+        d->data->br[cg][cr] = b;
+    } else if (d->resolveMask != newResolveMask) {
         detach();
-        d->br[cg][cr] = b;
-        d->resolveMask |= ResolveMask(1) << bitPosition(cg, cr);
     }
+
+    d->resolveMask = newResolveMask;
 }
 
 /*!
@@ -829,17 +859,14 @@ bool QPalette::isBrushSet(ColorGroup cg, ColorRole cr) const
 void QPalette::detach()
 {
     if (d->ref.loadRelaxed() != 1) {
-        QPalettePrivate *x = new QPalettePrivate;
-        for(int grp = 0; grp < (int)NColorGroups; grp++) {
-            for(int role = 0; role < (int)NColorRoles; role++)
-                x->br[grp][role] = d->br[grp][role];
-        }
+        QPalettePrivate *x = new QPalettePrivate(d->data);
         x->resolveMask = d->resolveMask;
         if (!d->ref.deref())
             delete d;
         d = x;
+    } else {
+        d->detach_no = ++QPalettePrivate::qt_palette_private_count;
     }
-    ++d->detach_no;
 }
 
 /*!
@@ -865,11 +892,11 @@ void QPalette::detach()
 */
 bool QPalette::operator==(const QPalette &p) const
 {
-    if (isCopyOf(p))
+    if (isCopyOf(p) || d->data == p.d->data)
         return true;
     for(int grp = 0; grp < (int)NColorGroups; grp++) {
         for(int role = 0; role < (int)NColorRoles; role++) {
-            if (d->br[grp][role] != p.d->br[grp][role])
+            if (d->data->br[grp][role] != p.d->data->br[grp][role])
                 return false;
         }
     }
@@ -903,7 +930,7 @@ bool QPalette::isEqual(QPalette::ColorGroup group1, QPalette::ColorGroup group2)
     if (group1 == group2)
         return true;
     for(int role = 0; role < (int)NColorRoles; role++) {
-        if (d->br[group1][role] != d->br[group2][role])
+        if (d->data->br[group1][role] != d->data->br[group2][role])
                 return false;
     }
     return true;
@@ -918,7 +945,18 @@ bool QPalette::isEqual(QPalette::ColorGroup group1, QPalette::ColorGroup group2)
 */
 qint64 QPalette::cacheKey() const
 {
-    return (((qint64) d->ser_no) << 32) | ((qint64) (d->detach_no));
+    return (((qint64) d->data->ser_no) << 32) | ((qint64) (d->detach_no));
+}
+
+static constexpr QPalette::ResolveMask allResolveMask()
+{
+    QPalette::ResolveMask mask = {0};
+    for (int role = 0; role < int(QPalette::NColorRoles); ++role) {
+        for (int grp = 0; grp < int(QPalette::NColorGroups); ++grp) {
+            mask |= (QPalette::ResolveMask(1) << bitPosition(QPalette::ColorGroup(grp), QPalette::ColorRole(role)));
+        }
+    }
+    return mask;
 }
 
 /*!
@@ -930,17 +968,21 @@ QPalette QPalette::resolve(const QPalette &other) const
     if ((*this == other && d->resolveMask == other.d->resolveMask)
         || d->resolveMask == 0) {
         QPalette o = other;
-        o.d->resolveMask = d->resolveMask;
+        o.setResolveMask(d->resolveMask);
         return o;
     }
 
+    if (d->resolveMask == allResolveMask())
+        return *this;
+
     QPalette palette(*this);
     palette.detach();
 
     for (int role = 0; role < int(NColorRoles); ++role) {
         for (int grp = 0; grp < int(NColorGroups); ++grp) {
             if (!(d->resolveMask & (ResolveMask(1) << bitPosition(ColorGroup(grp), ColorRole(role))))) {
-                palette.d->br[grp][role] = other.d->br[grp][role];
+                palette.d->data.detach();
+                palette.d->data->br[grp][role] = other.d->data->br[grp][role];
             }
         }
     }
@@ -1003,7 +1045,7 @@ QDataStream &operator<<(QDataStream &s, const QPalette &p)
         if (s.version() == 1) {
             // Qt 1.x
             for (int i = 0; i < NumOldRoles; ++i)
-                s << p.d->br[grp][oldRoles[i]].color();
+                s << p.d->data->br[grp][oldRoles[i]].color();
         } else {
             int max = (int)QPalette::NColorRoles;
             if (s.version() <= QDataStream::Qt_2_1)
@@ -1013,7 +1055,7 @@ QDataStream &operator<<(QDataStream &s, const QPalette &p)
             else if (s.version() <= QDataStream::Qt_5_11)
                 max = QPalette::ToolTipText + 1;
             for (int r = 0; r < max; r++)
-                s << p.d->br[grp][r];
+                s << p.d->data->br[grp][r];
         }
     }
     return s;
@@ -1168,47 +1210,6 @@ void QPalette::setColorGroup(ColorGroup cg, const QBrush &foreground, const QBru
     setBrush(cg, ToolTipText, toolTipText);
 }
 
-Q_GUI_EXPORT QPalette qt_fusionPalette()
-{
-    QColor backGround(239, 239, 239);
-    QColor light = backGround.lighter(150);
-    QColor mid(backGround.darker(130));
-    QColor midLight = mid.lighter(110);
-    QColor base = Qt::white;
-    QColor disabledBase(backGround);
-    QColor dark = backGround.darker(150);
-    QColor darkDisabled = QColor(209, 209, 209).darker(110);
-    QColor text = Qt::black;
-    QColor hightlightedText = Qt::white;
-    QColor disabledText = QColor(190, 190, 190);
-    QColor button = backGround;
-    QColor shadow = dark.darker(135);
-    QColor disabledShadow = shadow.lighter(150);
-    QColor placeholder = text;
-    placeholder.setAlpha(128);
-
-    QPalette fusionPalette(Qt::black,backGround,light,dark,mid,text,base);
-    fusionPalette.setBrush(QPalette::Midlight, midLight);
-    fusionPalette.setBrush(QPalette::Button, button);
-    fusionPalette.setBrush(QPalette::Shadow, shadow);
-    fusionPalette.setBrush(QPalette::HighlightedText, hightlightedText);
-
-    fusionPalette.setBrush(QPalette::Disabled, QPalette::Text, disabledText);
-    fusionPalette.setBrush(QPalette::Disabled, QPalette::WindowText, disabledText);
-    fusionPalette.setBrush(QPalette::Disabled, QPalette::ButtonText, disabledText);
-    fusionPalette.setBrush(QPalette::Disabled, QPalette::Base, disabledBase);
-    fusionPalette.setBrush(QPalette::Disabled, QPalette::Dark, darkDisabled);
-    fusionPalette.setBrush(QPalette::Disabled, QPalette::Shadow, disabledShadow);
-
-    fusionPalette.setBrush(QPalette::Active, QPalette::Highlight, QColor(48, 140, 198));
-    fusionPalette.setBrush(QPalette::Inactive, QPalette::Highlight, QColor(48, 140, 198));
-    fusionPalette.setBrush(QPalette::Disabled, QPalette::Highlight, QColor(145, 145, 145));
-
-    fusionPalette.setBrush(QPalette::PlaceholderText, placeholder);
-
-    return fusionPalette;
-}
-
 #ifndef QT_NO_DEBUG_STREAM
 static QString groupsToString(const QPalette &p, QPalette::ColorRole cr)
 {
@@ -1220,8 +1221,8 @@ static QString groupsToString(const QPalette &p, QPalette::ColorRole cr)
 
         if (p.isBrushSet(cg, cr)) {
             const auto &color = p.color(cg, cr);
-            groupString += QString::fromUtf8(groupEnum.valueToKey(cg)) + QLatin1Char(':') +
-                           color.name(QColor::HexArgb) + QLatin1Char(',');
+            groupString += QString::fromUtf8(groupEnum.valueToKey(cg)) + u':' +
+                           color.name(QColor::HexArgb) + u',';
         }
     }
     groupString.chop(1);
@@ -1265,3 +1266,5 @@ QDebug operator<<(QDebug dbg, const QPalette &p)
 #endif
 
 QT_END_NAMESPACE
+
+#include "moc_qpalette.cpp"
