@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qshortcut.h"
 #include "qshortcut_p.h"
@@ -163,14 +127,14 @@ void QShortcutPrivate::redoGrab(QShortcutMap &map)
         return;
     }
 
-    for (int id : qAsConst(sc_ids))
+    for (int id : std::as_const(sc_ids))
         map.removeShortcut(id, q);
 
     sc_ids.clear();
     if (sc_sequences.isEmpty())
         return;
-    sc_ids.reserve(sc_sequences.count());
-    for (const auto &keySequence : qAsConst(sc_sequences)) {
+    sc_ids.reserve(sc_sequences.size());
+    for (const auto &keySequence : std::as_const(sc_sequences)) {
         if (keySequence.isEmpty())
             continue;
         int id = map.addShortcut(q, keySequence, sc_context, contextMatcher());
@@ -373,7 +337,7 @@ QShortcut::~QShortcut()
 {
     Q_D(QShortcut);
     if (qApp) {
-         for (int id : qAsConst(d->sc_ids))
+         for (int id : std::as_const(d->sc_ids))
              QGuiApplicationPrivate::instance()->shortcutMap.removeShortcut(id, this);
     }
 }
