@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <qstylehints.h>
 #include <qpa/qplatformintegration.h>
@@ -107,11 +71,10 @@ public:
     An object of this class, obtained from QGuiApplication, provides access to certain global
     user interface parameters of the current platform.
 
-    Access is read only; typically the platform itself provides the user a way to tune these
-    parameters.
-
-    Access to these parameters are useful when implementing custom user interface components, in that
-    they allow the components to exhibit the same behaviour and feel as other components.
+    Access to most settings is read only. The platform itself usually provides the user with
+    ways to tune these parameters. Authors of custom user interface components should read
+    relevant settings to allow the components to exhibit the same behavior and feel as other
+    components.
 
     \sa QGuiApplication::styleHints()
  */
@@ -621,3 +584,5 @@ int QStyleHints::mouseQuickSelectionThreshold() const
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qstylehints.cpp"
