@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qwindow.h"
 
@@ -54,7 +18,7 @@
 
 #include "qwindow_p.h"
 #include "qguiapplication_p.h"
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 #  include "qaccessible.h"
 #endif
 #include "qhighdpiscaling_p.h"
@@ -232,12 +196,21 @@ QWindow::~QWindow()
         QGuiApplicationPrivate::currentMouseWindow = nullptr;
     if (QGuiApplicationPrivate::currentMousePressWindow == this)
         QGuiApplicationPrivate::currentMousePressWindow = nullptr;
+
+    d->isWindow = false;
 }
 
+QWindowPrivate::QWindowPrivate()
+    = default;
+
+QWindowPrivate::~QWindowPrivate()
+    = default;
+
 void QWindowPrivate::init(QScreen *targetScreen)
 {
     Q_Q(QWindow);
 
+    isWindow = true;
     parentWindow = static_cast<QWindow *>(q->QObject::parent());
 
     if (!parentWindow)
@@ -247,7 +220,6 @@ void QWindowPrivate::init(QScreen *targetScreen)
     // before the screen list is populated.
     if (Q_UNLIKELY(!parentWindow && !topLevelScreen)) {
         qFatal("Cannot create window: no screens available");
-        exit(1);
     }
     QGuiApplicationPrivate::window_list.prepend(q);
 
@@ -332,7 +304,6 @@ void QWindow::setVisibility(Visibility v)
         break;
     default:
         Q_ASSERT(false);
-        break;
     }
 }
 
@@ -1753,13 +1724,7 @@ void QWindow::setGeometry(const QRect &rect)
         QScreen *newScreen = d->screenForGeometry(rect);
         if (newScreen && isTopLevel())
             d->setTopLevelScreen(newScreen, true);
-
-        QRect nativeRect;
-        if (newScreen && isTopLevel())
-            nativeRect = QHighDpi::toNativePixels(rect, newScreen);
-        else
-            nativeRect = QHighDpi::toNativeLocalPosition(rect, newScreen);
-        d->platformWindow->setGeometry(nativeRect);
+        d->platformWindow->setGeometry(QHighDpi::toNativeWindowGeometry(rect, this));
     } else {
         d->geometry = rect;
 
@@ -2010,9 +1975,7 @@ void QWindowPrivate::destroy()
     // Unset platformWindow before deleting, so that the destructor of the
     // platform window does not recurse back into the platform window via
     // this window during destruction (e.g. as a result of platform events).
-    QPlatformWindow *pw = platformWindow;
-    platformWindow = nullptr;
-    delete pw;
+    delete std::exchange(platformWindow, nullptr);
 
     if (QGuiApplicationPrivate::focus_window == q)
         QGuiApplicationPrivate::focus_window = q->parent();
@@ -2273,8 +2236,42 @@ bool QWindow::close()
     if (!d->platformWindow)
         return true;
 
-    QBoolBlocker inCloseReset(d->inClose);
-    return d->platformWindow->close();
+    // The window might be deleted during close,
+    // as a result of delivering the close event.
+    QPointer guard(this);
+    d->inClose = true;
+    bool success = d->platformWindow->close();
+    if (guard)
+        d->inClose = false;
+
+    return success;
+}
+
+bool QWindowPrivate::participatesInLastWindowClosed() const
+{
+    Q_Q(const QWindow);
+
+    if (!q->isTopLevel())
+        return false;
+
+    // Tool-tip widgets do not normally have Qt::WA_QuitOnClose,
+    // but since we do not have a similar flag for non-widget
+    // windows we need an explicit exclusion here as well.
+    if (q->type() == Qt::ToolTip)
+        return false;
+
+    // A window with a transient parent is not a primary window,
+    // it's a secondary window.
+    if (q->transientParent())
+        return false;
+
+    return true;
+}
+
+bool QWindowPrivate::treatAsVisible() const
+{
+    Q_Q(const QWindow);
+    return q->isVisible();
 }
 
 /*!
@@ -2427,7 +2424,7 @@ bool QWindow::event(QEvent *ev)
 
     case QEvent::FocusIn: {
         focusInEvent(static_cast<QFocusEvent *>(ev));
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         QAccessible::State state;
         state.active = true;
         QAccessibleStateChangeEvent event(this, state);
@@ -2437,7 +2434,7 @@ bool QWindow::event(QEvent *ev)
 
     case QEvent::FocusOut: {
         focusOutEvent(static_cast<QFocusEvent *>(ev));
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         QAccessible::State state;
         state.active = true;
         QAccessibleStateChangeEvent event(this, state);
@@ -2451,20 +2448,25 @@ bool QWindow::event(QEvent *ev)
         break;
 #endif
 
-    case QEvent::Close:
+    case QEvent::Close: {
+
+        Q_D(QWindow);
+        const bool wasVisible = d->treatAsVisible();
+        const bool participatesInLastWindowClosed = d->participatesInLastWindowClosed();
+
+        // The window might be deleted in the close event handler
+        QPointer<QWindow> deletionGuard(this);
         closeEvent(static_cast<QCloseEvent*>(ev));
+
         if (ev->isAccepted()) {
-            Q_D(QWindow);
-            bool wasVisible = isVisible();
-            destroy();
-            if (wasVisible) {
-                // FIXME: This check for visibility is a workaround for both QWidgetWindow
-                // and QWindow having logic to emit lastWindowClosed, and possibly quit the
-                // application. We should find a better way to handle this.
-                d->maybeQuitOnLastWindowClosed();
-            }
+            if (deletionGuard)
+                destroy();
+            if (wasVisible && participatesInLastWindowClosed)
+                QGuiApplicationPrivate::instance()->maybeLastWindowClosed();
         }
+
         break;
+    }
 
     case QEvent::Expose:
         exposeEvent(static_cast<QExposeEvent *>(ev));
@@ -2732,7 +2734,7 @@ QPointF QWindow::mapToGlobal(const QPointF &pos) const
     // QTBUG-43252, prefer platform implementation for foreign windows.
     if (d->platformWindow
         && (d->platformWindow->isForeignWindow() || d->platformWindow->isEmbedded())) {
-        return QHighDpi::fromNativeLocalPosition(d->platformWindow->mapToGlobalF(QHighDpi::toNativeLocalPosition(pos, this)), this);
+        return QHighDpi::fromNativeGlobalPosition(d->platformWindow->mapToGlobalF(QHighDpi::toNativeLocalPosition(pos, this)), this);
     }
 
     if (!QHighDpiScaling::isActive())
@@ -2774,7 +2776,7 @@ QPointF QWindow::mapFromGlobal(const QPointF &pos) const
     // QTBUG-43252, prefer platform implementation for foreign windows.
     if (d->platformWindow
         && (d->platformWindow->isForeignWindow() || d->platformWindow->isEmbedded())) {
-        return QHighDpi::fromNativeLocalPosition(d->platformWindow->mapFromGlobalF(QHighDpi::toNativeLocalPosition(pos, this)), this);
+        return QHighDpi::fromNativeLocalPosition(d->platformWindow->mapFromGlobalF(QHighDpi::toNativeGlobalPosition(pos, this)), this);
     }
 
     if (!QHighDpiScaling::isActive())
@@ -2819,34 +2821,6 @@ Q_GUI_EXPORT QWindowPrivate *qt_window_private(QWindow *window)
     return window->d_func();
 }
 
-void QWindowPrivate::maybeQuitOnLastWindowClosed()
-{
-    if (!QCoreApplication::instance())
-        return;
-
-    Q_Q(QWindow);
-    if (!q->isTopLevel())
-        return;
-
-    QWindowList list = QGuiApplication::topLevelWindows();
-    bool lastWindowClosed = true;
-    for (int i = 0; i < list.size(); ++i) {
-        QWindow *w = list.at(i);
-        if (!w->isVisible() || w->transientParent() || w->type() == Qt::ToolTip)
-            continue;
-        lastWindowClosed = false;
-        break;
-    }
-    if (lastWindowClosed) {
-        QGuiApplicationPrivate::emitLastWindowClosed();
-
-        if (QGuiApplication::quitOnLastWindowClosed()) {
-            QCoreApplicationPrivate *applicationPrivate = static_cast<QCoreApplicationPrivate*>(QObjectPrivate::get(QCoreApplication::instance()));
-            applicationPrivate->maybeQuit();
-        }
-    }
-}
-
 QWindow *QWindowPrivate::topLevelWindow(QWindow::AncestorMode mode) const
 {
     Q_Q(const QWindow);
@@ -2864,13 +2838,6 @@ QWindow *QWindowPrivate::topLevelWindow(QWindow::AncestorMode mode) const
     return window;
 }
 
-#if QT_CONFIG(opengl)
-QOpenGLContext *QWindowPrivate::shareContext() const
-{
-    return qt_gl_global_share_context();
-};
-#endif
-
 /*!
     Creates a local representation of a window created by another process or by
     using native libraries below Qt.
