@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Copyright (C) 2013 Konstantin Ritt
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// Copyright (C) 2013 Konstantin Ritt
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qharfbuzzng_p.h"
 
@@ -242,7 +206,21 @@ static const hb_script_t _qtscript_to_hbscript[] = {
     HB_SCRIPT_CHORASMIAN,
     HB_SCRIPT_DIVES_AKURU,
     HB_SCRIPT_KHITAN_SMALL_SCRIPT,
-    HB_SCRIPT_YEZIDI
+    HB_SCRIPT_YEZIDI,
+#endif
+    // Unicode 14.0 additions (not present in harfbuzz-ng 2.9.1 and earlier)
+#if !HB_VERSION_ATLEAST(3, 0, 0)
+    hb_script_t(HB_TAG('C','p','m','n')), // Script_CyproMinoan
+    hb_script_t(HB_TAG('O','u','g','r')), // Script_OldUyghur
+    hb_script_t(HB_TAG('T','n','s','a')), // Script_Tangsa
+    hb_script_t(HB_TAG('T','o','t','o')), // Script_Toto
+    hb_script_t(HB_TAG('V','i','t','h')), // Script_Vithkuqi
+#else
+    HB_SCRIPT_CYPRO_MINOAN,
+    HB_SCRIPT_OLD_UYGHUR,
+    HB_SCRIPT_TANGSA,
+    HB_SCRIPT_TOTO,
+    HB_SCRIPT_VITHKUQI,
 #endif
 };
 static_assert(QChar::ScriptCount == sizeof(_qtscript_to_hbscript) / sizeof(_qtscript_to_hbscript[0]));
