@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qtextmarkdownimporter_p.h"
 #include "qtextdocumentfragment_p.h"
@@ -56,10 +20,12 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 Q_LOGGING_CATEGORY(lcMD, "qt.text.markdown")
 
-static const QChar Newline = QLatin1Char('\n');
-static const QChar Space = QLatin1Char(' ');
+static const QChar Newline = u'\n';
+static const QChar Space = u' ';
 
 // TODO maybe eliminate the margins after all views recognize BlockQuoteLevel, CSS can format it, etc.
 static const int BlockQuoteIndent = 40; // pixels, same as in QTextHtmlParserNode::initializeProperties
@@ -182,7 +148,7 @@ int QTextMarkdownImporter::cbEnterBlock(int blockType, void *det)
     switch (blockType) {
     case MD_BLOCK_P:
         if (!m_listStack.isEmpty())
-            qCDebug(lcMD, m_listItem ? "P of LI at level %d"  : "P continuation inside LI at level %d", int(m_listStack.count()));
+            qCDebug(lcMD, m_listItem ? "P of LI at level %d"  : "P continuation inside LI at level %d", int(m_listStack.size()));
         else
             qCDebug(lcMD, "P");
         m_needsInsertBlock = true;
@@ -194,9 +160,9 @@ int QTextMarkdownImporter::cbEnterBlock(int blockType, void *det)
     case MD_BLOCK_CODE: {
         MD_BLOCK_CODE_DETAIL *detail = static_cast<MD_BLOCK_CODE_DETAIL *>(det);
         m_codeBlock = true;
-        m_blockCodeLanguage = QLatin1String(detail->lang.text, int(detail->lang.size));
+        m_blockCodeLanguage = QLatin1StringView(detail->lang.text, int(detail->lang.size));
         m_blockCodeFence = detail->fence_char;
-        QString info = QLatin1String(detail->info.text, int(detail->info.size));
+        QString info = QLatin1StringView(detail->info.text, int(detail->info.size));
         m_needsInsertBlock = true;
         if (m_blockQuoteDepth)
             qCDebug(lcMD, "CODE lang '%s' info '%s' fenced with '%c' inside QUOTE %d", qPrintable(m_blockCodeLanguage), qPrintable(info), m_blockCodeFence, m_blockQuoteDepth);
@@ -236,7 +202,7 @@ int QTextMarkdownImporter::cbEnterBlock(int blockType, void *det)
             m_needsInsertList = true;
         MD_BLOCK_UL_DETAIL *detail = static_cast<MD_BLOCK_UL_DETAIL *>(det);
         m_listFormat = QTextListFormat();
-        m_listFormat.setIndent(m_listStack.count() + 1);
+        m_listFormat.setIndent(m_listStack.size() + 1);
         switch (detail->mark) {
         case '*':
             m_listFormat.setStyle(QTextListFormat::ListCircle);
@@ -248,7 +214,7 @@ int QTextMarkdownImporter::cbEnterBlock(int blockType, void *det)
             m_listFormat.setStyle(QTextListFormat::ListDisc);
             break;
         }
-        qCDebug(lcMD, "UL %c level %d", detail->mark, int(m_listStack.count()) + 1);
+        qCDebug(lcMD, "UL %c level %d", detail->mark, int(m_listStack.size()) + 1);
     } break;
     case MD_BLOCK_OL: {
         if (m_needsInsertList) // list nested in an empty list
@@ -257,10 +223,10 @@ int QTextMarkdownImporter::cbEnterBlock(int blockType, void *det)
             m_needsInsertList = true;
         MD_BLOCK_OL_DETAIL *detail = static_cast<MD_BLOCK_OL_DETAIL *>(det);
         m_listFormat = QTextListFormat();
-        m_listFormat.setIndent(m_listStack.count() + 1);
+        m_listFormat.setIndent(m_listStack.size() + 1);
         m_listFormat.setNumberSuffix(QChar::fromLatin1(detail->mark_delimiter));
         m_listFormat.setStyle(QTextListFormat::ListDecimal);
-        qCDebug(lcMD, "OL xx%d level %d", detail->mark_delimiter, int(m_listStack.count()) + 1);
+        qCDebug(lcMD, "OL xx%d level %d", detail->mark_delimiter, int(m_listStack.size()) + 1);
     } break;
     case MD_BLOCK_TD: {
         MD_BLOCK_TD_DETAIL *detail = static_cast<MD_BLOCK_TD_DETAIL *>(det);
@@ -331,7 +297,7 @@ int QTextMarkdownImporter::cbLeaveBlock(int blockType, void *detail)
         if (Q_UNLIKELY(m_listStack.isEmpty())) {
             qCWarning(lcMD, "list ended unexpectedly");
         } else {
-            qCDebug(lcMD, "list at level %d ended", int(m_listStack.count()));
+            qCDebug(lcMD, "list at level %d ended", int(m_listStack.size()));
             m_listStack.pop();
         }
         break;
@@ -368,7 +334,7 @@ int QTextMarkdownImporter::cbLeaveBlock(int blockType, void *detail)
         m_cursor->movePosition(QTextCursor::End);
         break;
     case MD_BLOCK_LI:
-        qCDebug(lcMD, "LI at level %d ended", int(m_listStack.count()));
+        qCDebug(lcMD, "LI at level %d ended", int(m_listStack.size()));
         m_listItem = false;
         break;
     case MD_BLOCK_CODE: {
@@ -532,6 +498,14 @@ int QTextMarkdownImporter::cbText(int textType, const char *text, unsigned size)
     case MD_BLOCK_TD:
         m_nonEmptyTableCells.append(m_tableCol);
         break;
+    case MD_BLOCK_CODE:
+        if (s == Newline) {
+            // defer a blank line until we see something else in the code block,
+            // to avoid ending every code block with a gratuitous blank line
+            m_needsInsertBlock = true;
+            s = QString();
+        }
+        break;
     default:
         break;
     }
@@ -559,18 +533,16 @@ int QTextMarkdownImporter::cbText(int textType, const char *text, unsigned size)
         QTextBlockFormat bfmt = m_cursor->blockFormat();
         QString debugInfo;
         if (m_cursor->currentList())
-            debugInfo = QLatin1String("in list at depth ") + QString::number(m_cursor->currentList()->format().indent());
+            debugInfo = "in list at depth "_L1 + QString::number(m_cursor->currentList()->format().indent());
         if (bfmt.hasProperty(QTextFormat::BlockQuoteLevel))
-            debugInfo += QLatin1String("in blockquote at depth ") +
+            debugInfo += "in blockquote at depth "_L1 +
                     QString::number(bfmt.intProperty(QTextFormat::BlockQuoteLevel));
         if (bfmt.hasProperty(QTextFormat::BlockCodeLanguage))
-            debugInfo += QLatin1String("in a code block");
+            debugInfo += "in a code block"_L1;
         qCDebug(lcMD) << textType << "in block" << m_blockType << s << qPrintable(debugInfo)
                       << "bindent" << bfmt.indent() << "tindent" << bfmt.textIndent()
                       << "margins" << bfmt.leftMargin() << bfmt.topMargin() << bfmt.bottomMargin() << bfmt.rightMargin();
     }
-    qCDebug(lcMD) << textType << "in block" << m_blockType << s << "in list?" << m_cursor->currentList()
-                  << "indent" << m_cursor->blockFormat().indent();
     return 0; // no error
 }
 
@@ -605,8 +577,10 @@ void QTextMarkdownImporter::insertBlock()
     }
     if (m_codeBlock) {
         blockFormat.setProperty(QTextFormat::BlockCodeLanguage, m_blockCodeLanguage);
-        if (m_blockCodeFence)
+        if (m_blockCodeFence) {
+            blockFormat.setNonBreakableLines(true);
             blockFormat.setProperty(QTextFormat::BlockCodeFence, QString(QLatin1Char(m_blockCodeFence)));
+        }
         charFormat.setFont(m_monoFont);
     } else {
         blockFormat.setTopMargin(m_paragraphMargin);
@@ -617,7 +591,7 @@ void QTextMarkdownImporter::insertBlock()
     else
         blockFormat.setMarker(m_markerType);
     if (!m_listStack.isEmpty())
-        blockFormat.setIndent(m_listStack.count());
+        blockFormat.setIndent(m_listStack.size());
     if (m_doc->isEmpty()) {
         m_cursor->setBlockFormat(blockFormat);
         m_cursor->setCharFormat(charFormat);
