@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qtextmarkdownwriter_p.h"
 #include "qtextdocumentlayout_p.h"
@@ -53,17 +17,19 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 Q_LOGGING_CATEGORY(lcMDW, "qt.text.markdown.writer")
 
-static const QChar Space = QLatin1Char(' ');
-static const QChar Tab = QLatin1Char('\t');
-static const QChar Newline = QLatin1Char('\n');
-static const QChar CarriageReturn = QLatin1Char('\r');
+static const QChar Space = u' ';
+static const QChar Tab = u'\t';
+static const QChar Newline = u'\n';
+static const QChar CarriageReturn = u'\r';
 static const QChar LineBreak = u'\x2028';
-static const QChar DoubleQuote = QLatin1Char('"');
-static const QChar Backtick = QLatin1Char('`');
-static const QChar Backslash = QLatin1Char('\\');
-static const QChar Period = QLatin1Char('.');
+static const QChar DoubleQuote = u'"';
+static const QChar Backtick = u'`';
+static const QChar Backslash = u'\\';
+static const QChar Period = u'.';
 
 QTextMarkdownWriter::QTextMarkdownWriter(QTextStream &stream, QTextDocument::MarkdownFeatures features)
   : m_stream(stream), m_features(features)
@@ -81,28 +47,28 @@ void QTextMarkdownWriter::writeTable(const QAbstractItemModel *table)
 {
     QList<int> tableColumnWidths(table->columnCount());
     for (int col = 0; col < table->columnCount(); ++col) {
-        tableColumnWidths[col] = table->headerData(col, Qt::Horizontal).toString().length();
+        tableColumnWidths[col] = table->headerData(col, Qt::Horizontal).toString().size();
         for (int row = 0; row < table->rowCount(); ++row) {
             tableColumnWidths[col] = qMax(tableColumnWidths[col],
-                table->data(table->index(row, col)).toString().length());
+                table->data(table->index(row, col)).toString().size());
         }
     }
 
     // write the header and separator
     for (int col = 0; col < table->columnCount(); ++col) {
         QString s = table->headerData(col, Qt::Horizontal).toString();
-        m_stream << "|" << s << QString(tableColumnWidths[col] - s.length(), Space);
+        m_stream << "|" << s << QString(tableColumnWidths[col] - s.size(), Space);
     }
     m_stream << "|" << Qt::endl;
-    for (int col = 0; col < tableColumnWidths.length(); ++col)
-        m_stream << '|' << QString(tableColumnWidths[col], QLatin1Char('-'));
+    for (int col = 0; col < tableColumnWidths.size(); ++col)
+        m_stream << '|' << QString(tableColumnWidths[col], u'-');
     m_stream << '|'<< Qt::endl;
 
     // write the body
     for (int row = 0; row < table->rowCount(); ++row) {
         for (int col = 0; col < table->columnCount(); ++col) {
             QString s = table->data(table->index(row, col)).toString();
-            m_stream << "|" << s << QString(tableColumnWidths[col] - s.length(), Space);
+            m_stream << "|" << s << QString(tableColumnWidths[col] - s.size(), Space);
         }
         m_stream << '|'<< Qt::endl;
     }
@@ -129,7 +95,7 @@ void QTextMarkdownWriter::writeFrame(const QTextFrame *frame)
                 while (it != cell.end()) {
                     QTextBlock block = it.currentBlock();
                     if (block.isValid())
-                        cellTextLen += block.text().length();
+                        cellTextLen += block.text().size();
                     ++it;
                 }
                 if (cell.columnSpan() == 1 && tableColumnWidths[col] < cellTextLen)
@@ -165,8 +131,8 @@ void QTextMarkdownWriter::writeFrame(const QTextFrame *frame)
                 if (tableRow < cell.row()) {
                     if (tableRow == 0) {
                         m_stream << Newline;
-                        for (int col = 0; col < tableColumnWidths.length(); ++col)
-                            m_stream << '|' << QString(tableColumnWidths[col], QLatin1Char('-'));
+                        for (int col = 0; col < tableColumnWidths.size(); ++col)
+                            m_stream << '|' << QString(tableColumnWidths[col], u'-');
                         m_stream << '|';
                     }
                     m_stream << Newline << "|";
@@ -190,11 +156,11 @@ void QTextMarkdownWriter::writeFrame(const QTextFrame *frame)
                 for (int col = cell.column(); col < spanEndCol; ++col)
                     m_stream << "|";
             } else if (m_fencedCodeBlock && ending) {
-                m_stream << m_linePrefix << QString(m_wrappedLineIndent, Space)
+                m_stream << Newline << m_linePrefix << QString(m_wrappedLineIndent, Space)
                          << m_codeBlockFence << Newline << Newline;
                 m_codeBlockFence.clear();
             } else if (m_indentedCodeBlock && nextIsDifferent) {
-                m_stream << Newline;
+                m_stream << Newline << Newline;
             } else if (endingCol > 0) {
                 if (block.textList() || block.blockFormat().hasProperty(QTextFormat::BlockCodeLanguage)) {
                     m_stream << Newline;
@@ -247,16 +213,16 @@ QTextMarkdownWriter::ListInfo QTextMarkdownWriter::listInfo(QTextList *list)
 
 static int nearestWordWrapIndex(const QString &s, int before)
 {
-    before = qMin(before, s.length());
+    before = qMin(before, s.size());
     int fragBegin = qMax(before - 15, 0);
     if (lcMDW().isDebugEnabled()) {
         QString frag = s.mid(fragBegin, 30);
         qCDebug(lcMDW) << frag << before;
-        qCDebug(lcMDW) << QString(before - fragBegin, Period) + QLatin1Char('<');
+        qCDebug(lcMDW) << QString(before - fragBegin, Period) + u'<';
     }
     for (int i = before - 1; i >= 0; --i) {
         if (s.at(i).isSpace()) {
-            qCDebug(lcMDW) << QString(i - fragBegin, Period) + QLatin1Char('^') << i;
+            qCDebug(lcMDW) << QString(i - fragBegin, Period) + u'^' << i;
             return i;
         }
     }
@@ -266,7 +232,7 @@ static int nearestWordWrapIndex(const QString &s, int before)
 
 static int adjacentBackticksCount(const QString &s)
 {
-    int start = -1, len = s.length();
+    int start = -1, len = s.size();
     int ret = 0;
     for (int i = 0; i < len; ++i) {
         if (s.at(i) == Backtick) {
@@ -290,7 +256,7 @@ static void maybeEscapeFirstChar(QString &s)
     char firstChar = sTrimmed.at(0).toLatin1();
     if (firstChar == '*' || firstChar == '+' || firstChar == '-') {
         int i = s.indexOf(QLatin1Char(firstChar));
-        s.insert(i, QLatin1Char('\\'));
+        s.insert(i, u'\\');
     }
 }
 
@@ -368,7 +334,8 @@ int QTextMarkdownWriter::writeBlock(const QTextBlock &block, bool wrap, bool ign
     QTextBlockFormat blockFmt = block.blockFormat();
     bool missedBlankCodeBlockLine = false;
     const bool codeBlock = blockFmt.hasProperty(QTextFormat::BlockCodeFence) ||
-            blockFmt.stringProperty(QTextFormat::BlockCodeLanguage).length() > 0;
+            blockFmt.stringProperty(QTextFormat::BlockCodeLanguage).size() > 0 ||
+            blockFmt.nonBreakableLines();
     if (m_fencedCodeBlock && !codeBlock) {
         m_stream << m_linePrefix << m_codeBlockFence << Newline;
         m_fencedCodeBlock = false;
@@ -424,11 +391,11 @@ int QTextMarkdownWriter::writeBlock(const QTextBlock &block, bool wrap, bool ign
             if (suffix.isEmpty())
                 suffix = QString(Period);
             QString numberStr = QString::number(number) + suffix + Space;
-            if (numberStr.length() == 3)
+            if (numberStr.size() == 3)
                 numberStr += Space;
             prefix += numberStr;
         } else {
-            prefix += QLatin1String(bullet) + Space;
+            prefix += QLatin1StringView(bullet) + Space;
         }
         m_stream << prefix;
     } else if (blockFmt.hasProperty(QTextFormat::BlockTrailingHorizontalRulerWidth)) {
@@ -442,7 +409,7 @@ int QTextMarkdownWriter::writeBlock(const QTextBlock &block, bool wrap, bool ign
         if (!m_fencedCodeBlock) {
             QString fenceChar = blockFmt.stringProperty(QTextFormat::BlockCodeFence);
             if (fenceChar.isEmpty())
-                fenceChar = QLatin1String("`");
+                fenceChar = "`"_L1;
             m_codeBlockFence = QString(3, fenceChar.at(0));
             if (blockFmt.hasProperty(QTextFormat::BlockIndent))
                 m_codeBlockFence = QString(m_wrappedLineIndent, Space) + m_codeBlockFence;
@@ -477,7 +444,7 @@ int QTextMarkdownWriter::writeBlock(const QTextBlock &block, bool wrap, bool ign
     // It would be convenient if QTextStream had a lineCharPos() accessor,
     // to keep track of how many characters (not bytes) have been written on the current line,
     // but it doesn't.  So we have to keep track with this col variable.
-    int col = wrapIndentString.length();
+    int col = wrapIndentString.size();
     bool mono = false;
     bool startsOrEndsWithBacktick = false;
     bool bold = false;
@@ -504,32 +471,38 @@ int QTextMarkdownWriter::writeBlock(const QTextBlock &block, bool wrap, bool ign
             QTextImageFormat ifmt = fmt.toImageFormat();
             QString desc = ifmt.stringProperty(QTextFormat::ImageAltText);
             if (desc.isEmpty())
-                desc = QLatin1String("image");
-            QString s = QLatin1String("![") + desc + QLatin1String("](") + ifmt.name();
+                desc = "image"_L1;
+            QString s = "!["_L1 + desc + "]("_L1 + ifmt.name();
             QString title = ifmt.stringProperty(QTextFormat::ImageTitle);
             if (!title.isEmpty())
                 s += Space + DoubleQuote + title + DoubleQuote;
-            s += QLatin1Char(')');
-            if (wrap && col + s.length() > ColumnLimit) {
+            s += u')';
+            if (wrap && col + s.size() > ColumnLimit) {
                 m_stream << Newline << wrapIndentString;
                 col = m_wrappedLineIndent;
             }
             m_stream << s;
-            col += s.length();
+            col += s.size();
         } else if (fmt.hasProperty(QTextFormat::AnchorHref)) {
-            QString s = QLatin1Char('[') + fragmentText + QLatin1String("](") +
-                    fmt.property(QTextFormat::AnchorHref).toString();
-            if (fmt.hasProperty(QTextFormat::TextToolTip)) {
-                s += Space;
-                s += createLinkTitle(fmt.property(QTextFormat::TextToolTip).toString());
+            const auto href = fmt.property(QTextFormat::AnchorHref).toString();
+            const bool hasToolTip = fmt.hasProperty(QTextFormat::TextToolTip);
+            QString s;
+            if (!hasToolTip && href == fragmentText && !QUrl(href, QUrl::StrictMode).scheme().isEmpty()) {
+                s = u'<' + href + u'>';
+            } else {
+                s = u'[' + fragmentText + "]("_L1 + href;
+                if (hasToolTip) {
+                    s += Space;
+                    s += createLinkTitle(fmt.property(QTextFormat::TextToolTip).toString());
+                }
+                s += u')';
             }
-            s += QLatin1Char(')');
-            if (wrap && col + s.length() > ColumnLimit) {
+            if (wrap && col + s.size() > ColumnLimit) {
                 m_stream << Newline << wrapIndentString;
                 col = m_wrappedLineIndent;
             }
             m_stream << s;
-            col += s.length();
+            col += s.size();
         } else {
             QFontInfo fontInfo(fmt.font());
             bool monoFrag = fontInfo.fixedPitch() || fmt.fontFixedPitch();
@@ -545,35 +518,35 @@ int QTextMarkdownWriter::writeBlock(const QTextBlock &block, bool wrap, bool ign
                 }
                 if (!blockFmt.headingLevel() && !mono) {
                     if (fontInfo.bold() != bold) {
-                        markers += QLatin1String("**");
+                        markers += "**"_L1;
                         bold = fontInfo.bold();
                     }
                     if (fontInfo.italic() != italic) {
-                        markers += QLatin1Char('*');
+                        markers += u'*';
                         italic = fontInfo.italic();
                     }
                     if (fontInfo.strikeOut() != strikeOut) {
-                        markers += QLatin1String("~~");
+                        markers += "~~"_L1;
                         strikeOut = fontInfo.strikeOut();
                     }
                     if (fontInfo.underline() != underline) {
                         // Markdown doesn't support underline, but the parser will treat a single underline
                         // the same as a single asterisk, and the marked fragment will be rendered in italics.
                         // That will have to do.
-                        markers += QLatin1Char('_');
+                        markers += u'_';
                         underline = fontInfo.underline();
                     }
                 }
             }
-            if (wrap && col + markers.length() * 2 + fragmentText.length() > ColumnLimit) {
+            if (wrap && col + markers.size() * 2 + fragmentText.size() > ColumnLimit) {
                 int i = 0;
-                int fragLen = fragmentText.length();
+                const int fragLen = fragmentText.size();
                 bool breakingLine = false;
                 while (i < fragLen) {
                     if (col >= ColumnLimit) {
                         m_stream << Newline << wrapIndentString;
                         col = m_wrappedLineIndent;
-                        while (fragmentText[i].isSpace())
+                        while (i < fragLen && fragmentText[i].isSpace())
                             ++i;
                     }
                     int j = i + ColumnLimit - col;
@@ -592,7 +565,7 @@ int QTextMarkdownWriter::writeBlock(const QTextBlock &block, bool wrap, bool ign
                     QString subfrag = fragmentText.mid(i, j - i);
                     if (!i) {
                         m_stream << markers;
-                        col += markers.length();
+                        col += markers.size();
                     }
                     if (col == m_wrappedLineIndent)
                         maybeEscapeFirstChar(subfrag);
@@ -601,13 +574,13 @@ int QTextMarkdownWriter::writeBlock(const QTextBlock &block, bool wrap, bool ign
                         m_stream << Newline << wrapIndentString;
                         col = m_wrappedLineIndent;
                     } else {
-                        col += subfrag.length();
+                        col += subfrag.size();
                     }
                     i = j + 1;
                 }
             } else {
                 m_stream << markers << fragmentText;
-                col += markers.length() + fragmentText.length();
+                col += markers.size() + fragmentText.size();
             }
         }
     }
