@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2014 BogDan Vatra <bogdan@kde.org>
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2014 BogDan Vatra <bogdan@kde.org>
+// Copyright (C) 2022 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <dlfcn.h>
 #include <pthread.h>
@@ -63,6 +27,7 @@
 #include <QtCore/qbasicatomic.h>
 #include <QtCore/qjnienvironment.h>
 #include <QtCore/qjniobject.h>
+#include <QtCore/qprocess.h>
 #include <QtCore/qresource.h>
 #include <QtCore/qthread.h>
 #include <QtGui/private/qguiapplication_p.h>
@@ -79,9 +44,9 @@ static jmethodID m_loadClassMethodID = nullptr;
 static AAssetManager *m_assetManager = nullptr;
 static jobject m_assets = nullptr;
 static jobject m_resourcesObj = nullptr;
-static jobject m_activityObject = nullptr;
+static QtJniTypes::Activity m_activityObject = nullptr;
 static jmethodID m_createSurfaceMethodID = nullptr;
-static jobject m_serviceObject = nullptr;
+static QtJniTypes::Service m_serviceObject = nullptr;
 static jmethodID m_setSurfaceGeometryMethodID = nullptr;
 static jmethodID m_destroySurfaceMethodID = nullptr;
 
@@ -195,12 +160,12 @@ namespace QtAndroid
         return m_applicationClass;
     }
 
-    jobject activity()
+    QtJniTypes::Activity activity()
     {
         return m_activityObject;
     }
 
-    jobject service()
+    QtJniTypes::Service service()
     {
         return m_serviceObject;
     }
@@ -210,9 +175,10 @@ namespace QtAndroid
         QJniObject::callStaticMethod<void>(m_applicationClass, "setSystemUiVisibility", "(I)V", jint(uiVisibility));
     }
 
-    void notifyAccessibilityLocationChange()
+    void notifyAccessibilityLocationChange(uint accessibilityObjectId)
     {
-        QJniObject::callStaticMethod<void>(m_applicationClass, "notifyAccessibilityLocationChange");
+        QJniObject::callStaticMethod<void>(m_applicationClass, "notifyAccessibilityLocationChange",
+                                           "(I)V", accessibilityObjectId);
     }
 
     void notifyObjectHide(uint accessibilityObjectId, uint parentObjectId)
@@ -232,6 +198,12 @@ namespace QtAndroid
                                            "(ILjava/lang/String;)V", accessibilityObjectId, value);
     }
 
+    void notifyScrolledEvent(uint accessibilityObjectId)
+    {
+        QJniObject::callStaticMethod<void>(m_applicationClass, "notifyScrolledEvent", "(I)V",
+                                           accessibilityObjectId);
+    }
+
     void notifyQtAndroidPluginRunning(bool running)
     {
         QJniObject::callStaticMethod<void>(m_applicationClass, "notifyQtAndroidPluginRunning","(Z)V", running);
@@ -328,7 +300,7 @@ namespace QtAndroid
         QString manufacturer = QJniObject::getStaticObjectField("android/os/Build", "MANUFACTURER", "Ljava/lang/String;").toString();
         QString model = QJniObject::getStaticObjectField("android/os/Build", "MODEL", "Ljava/lang/String;").toString();
 
-        return manufacturer + QLatin1Char(' ') + model;
+        return manufacturer + u' ' + model;
     }
 
     jint generateViewId()
@@ -472,29 +444,19 @@ namespace QtAndroid
 
 } // namespace QtAndroid
 
-static jboolean startQtAndroidPlugin(JNIEnv *env, jobject /*object*/, jstring paramsString, jstring environmentString)
+static jboolean startQtAndroidPlugin(JNIEnv *env, jobject /*object*/, jstring paramsString)
 {
     m_androidPlatformIntegration = nullptr;
     m_androidAssetsFileEngineHandler = new AndroidAssetsFileEngineHandler();
     m_androidContentFileEngineHandler = new AndroidContentFileEngineHandler();
     m_mainLibraryHnd = nullptr;
-    { // Set env. vars
-        const char *nativeString = env->GetStringUTFChars(environmentString, 0);
-        const QList<QByteArray> envVars = QByteArray(nativeString).split('\t');
-        env->ReleaseStringUTFChars(environmentString, nativeString);
-        for (const QByteArray &envVar : envVars) {
-            int pos = envVar.indexOf('=');
-            if (pos != -1 && ::setenv(envVar.left(pos), envVar.mid(pos + 1), 1) != 0)
-                qWarning() << "Can't set environment" << envVar;
-        }
-    }
 
     const char *nativeString = env->GetStringUTFChars(paramsString, 0);
-    QByteArray string = nativeString;
+    const QStringList argsList = QProcess::splitCommand(QString::fromUtf8(nativeString));
     env->ReleaseStringUTFChars(paramsString, nativeString);
 
-    for (auto str : string.split('\t'))
-        m_applicationParams.append(str.split(' '));
+    for (const QString &arg : argsList)
+        m_applicationParams.append(arg.toUtf8());
 
     // Go home
     QDir::setCurrent(QDir::homePath());
@@ -551,16 +513,21 @@ static void startQtApplication(JNIEnv */*env*/, jclass /*clazz*/)
             vm->AttachCurrentThread(&env, &args);
     }
 
+    // Register type for invokeMethod() calls.
+    qRegisterMetaType<Qt::ScreenOrientation>("Qt::ScreenOrientation");
+
     // Register resources if they are available
     if (QFile{QStringLiteral("assets:/android_rcc_bundle.rcc")}.exists())
         QResource::registerResource(QStringLiteral("assets:/android_rcc_bundle.rcc"));
 
-    QVarLengthArray<const char *> params(m_applicationParams.size());
-    for (int i = 0; i < m_applicationParams.size(); i++)
-        params[i] = static_cast<const char *>(m_applicationParams[i].constData());
+    const int argc = m_applicationParams.size();
+    QVarLengthArray<char *> argv(argc + 1);
+    for (int i = 0; i < argc; i++)
+        argv[i] = m_applicationParams[i].data();
+    argv[argc] = nullptr;
 
     startQtAndroidPluginCalled.fetchAndAddRelease(1);
-    int ret = m_main(m_applicationParams.length(), const_cast<char **>(params.data()));
+    int ret = m_main(argc, argv.data());
 
     if (m_mainLibraryHnd) {
         int res = dlclose(m_mainLibraryHnd);
@@ -654,24 +621,29 @@ static void setDisplayMetrics(JNIEnv * /*env*/, jclass /*clazz*/, jint screenWid
                               jint availableHeightPixels, jdouble xdpi, jdouble ydpi,
                               jdouble scaledDensity, jdouble density, jfloat refreshRate)
 {
+    Q_UNUSED(availableLeftPixels)
+    Q_UNUSED(availableTopPixels)
+
     m_availableWidthPixels = availableWidthPixels;
     m_availableHeightPixels = availableHeightPixels;
     m_scaledDensity = scaledDensity;
     m_density = density;
 
+    const QSize screenSize(screenWidthPixels, screenHeightPixels);
+    // available geometry always starts from top left
+    const QRect availableGeometry(0, 0, availableWidthPixels, availableHeightPixels);
+    const QSize physicalSize(qRound(double(screenWidthPixels) / xdpi * 25.4),
+                             qRound(double(screenHeightPixels) / ydpi * 25.4));
+
     QMutexLocker lock(&m_platformMutex);
     if (!m_androidPlatformIntegration) {
         QAndroidPlatformIntegration::setDefaultDisplayMetrics(
-                availableLeftPixels, availableTopPixels, availableWidthPixels,
-                availableHeightPixels, qRound(double(screenWidthPixels) / xdpi * 25.4),
-                qRound(double(screenHeightPixels) / ydpi * 25.4), screenWidthPixels,
-                screenHeightPixels);
+                availableGeometry.left(), availableGeometry.top(), availableGeometry.width(),
+                availableGeometry.height(), physicalSize.width(), physicalSize.height(),
+                screenSize.width(), screenSize.height());
     } else {
-        m_androidPlatformIntegration->setPhysicalSize(qRound(double(screenWidthPixels)  / xdpi * 25.4),
-                                                      qRound(double(screenHeightPixels) / ydpi * 25.4));
-        m_androidPlatformIntegration->setScreenSize(screenWidthPixels, screenHeightPixels);
-        m_androidPlatformIntegration->setAvailableGeometry(QRect(availableLeftPixels, availableTopPixels,
-                                                                 availableWidthPixels, availableHeightPixels));
+        m_androidPlatformIntegration->setScreenSizeParameters(physicalSize, screenSize,
+                                                              availableGeometry);
         m_androidPlatformIntegration->setRefreshRate(refreshRate);
     }
 }
@@ -767,9 +739,13 @@ static void handleOrientationChanged(JNIEnv */*env*/, jobject /*thiz*/, jint new
     QAndroidPlatformIntegration::setScreenOrientation(screenOrientation, native);
     QMutexLocker lock(&m_platformMutex);
     if (m_androidPlatformIntegration) {
-        QPlatformScreen *screen = m_androidPlatformIntegration->screen();
-        QWindowSystemInterface::handleScreenOrientationChange(screen->screen(),
-                                                              screenOrientation);
+        QAndroidPlatformScreen *screen = m_androidPlatformIntegration->screen();
+        // Use invokeMethod to keep the certain order of the "geometry change"
+        // and "orientation change" event handling.
+        if (screen) {
+            QMetaObject::invokeMethod(screen, "setOrientation", Qt::AutoConnection,
+                                      Q_ARG(Qt::ScreenOrientation, screenOrientation));
+        }
     }
 }
 
@@ -779,6 +755,12 @@ static void handleRefreshRateChanged(JNIEnv */*env*/, jclass /*cls*/, jfloat ref
         m_androidPlatformIntegration->setRefreshRate(refreshRate);
 }
 
+static void handleUiDarkModeChanged(JNIEnv */*env*/, jobject /*thiz*/, jint newUiMode)
+{
+    QAndroidPlatformIntegration::setAppearance(
+        (newUiMode == 1 ) ? QPlatformTheme::Appearance::Dark : QPlatformTheme::Appearance::Light);
+}
+
 static void onActivityResult(JNIEnv */*env*/, jclass /*cls*/,
                              jint requestCode,
                              jint resultCode,
@@ -798,8 +780,7 @@ static jobject onBind(JNIEnv */*env*/, jclass /*cls*/, jobject intent)
 }
 
 static JNINativeMethod methods[] = {
-    { "startQtAndroidPlugin", "(Ljava/lang/String;Ljava/lang/String;)Z",
-      (void *)startQtAndroidPlugin },
+    { "startQtAndroidPlugin", "(Ljava/lang/String;)Z", (void *)startQtAndroidPlugin },
     { "startQtApplication", "()V", (void *)startQtApplication },
     { "quitQtAndroidPlugin", "()V", (void *)quitQtAndroidPlugin },
     { "quitQtCoreApplication", "()V", (void *)quitQtCoreApplication },
@@ -809,6 +790,7 @@ static JNINativeMethod methods[] = {
     { "setSurface", "(ILjava/lang/Object;II)V", (void *)setSurface },
     { "updateWindow", "()V", (void *)updateWindow },
     { "updateApplicationState", "(I)V", (void *)updateApplicationState },
+    { "handleUiDarkModeChanged", "(I)V", (void *)handleUiDarkModeChanged },
     { "handleOrientationChanged", "(II)V", (void *)handleOrientationChanged },
     { "onActivityResult", "(IILandroid/content/Intent;)V", (void *)onActivityResult },
     { "onNewIntent", "(Landroid/content/Intent;)V", (void *)onNewIntent },
