@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QtCore/qglobal.h>
 
@@ -63,9 +27,11 @@
 
 QT_USE_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 static NSString *strippedText(QString s)
 {
-    s.remove(QLatin1String("..."));
+    s.remove("..."_L1);
     return QPlatformTheme::removeMnemonics(s).trimmed().toNSString();
 }
 
@@ -259,7 +225,12 @@ typedef QSharedPointer<QFileDialogOptions> SharedPointerFileDialogOptions;
         }
     }
 
-    QString qtFileName = QFileInfo(QString::fromNSString(filename)).fileName();
+    // Treat symbolic links and aliases to directories like directories
+    QFileInfo fileInfo(QString::fromNSString(filename));
+    if (fileInfo.isSymLink() && QFileInfo(fileInfo.symLinkTarget()).isDir())
+        return YES;
+
+    QString qtFileName = fileInfo.fileName();
     // No filter means accept everything
     bool nameMatches = m_selectedNameFilter->isEmpty();
     // Check if the current file name filter accepts the file:
@@ -285,7 +256,7 @@ typedef QSharedPointer<QFileDialogOptions> SharedPointerFileDialogOptions;
             return NO;
     }
     if (!(filter & QDir::Hidden)
-        && (qtFileName.startsWith(QLatin1Char('.')) || [self isHiddenFileAtURL:url]))
+        && (qtFileName.startsWith(u'.') || [self isHiddenFileAtURL:url]))
             return NO;
 
     return YES;
@@ -298,7 +269,7 @@ typedef QSharedPointer<QFileDialogOptions> SharedPointerFileDialogOptions;
     if (filters.size() > 0){
         for (int i = 0; i < filters.size(); ++i) {
             QString filter = hideDetails ? [self removeExtensions:filters.at(i)] : filters.at(i);
-            [m_popupButton addItemWithTitle:filter.toNSString()];
+            [m_popupButton.menu addItemWithTitle:filter.toNSString() action:nil keyEquivalent:@""];
         }
         [m_popupButton selectItemAtIndex:0];
         m_panel.accessoryView = m_accessoryView;
@@ -438,13 +409,13 @@ typedef QSharedPointer<QFileDialogOptions> SharedPointerFileDialogOptions;
 
     QStringList fileTypes;
     for (const QString &filter : *m_selectedNameFilter) {
-        if (!filter.startsWith(QLatin1String("*.")))
+        if (!filter.startsWith("*."_L1))
             continue;
 
-        if (filter.contains(QLatin1Char('?')))
+        if (filter.contains(u'?'))
             continue;
 
-        if (filter.count(QLatin1Char('*')) != 1)
+        if (filter.count(u'*') != 1)
             continue;
 
         auto extensions = filter.split('.', Qt::SkipEmptyParts);
@@ -492,7 +463,7 @@ typedef QSharedPointer<QFileDialogOptions> SharedPointerFileDialogOptions;
                 (filterToUse == -1 && currentFilter.startsWith(selectedFilter)))
                 filterToUse = i;
             QString filter = hideDetails ? [self removeExtensions:currentFilter] : currentFilter;
-            [m_popupButton addItemWithTitle:filter.toNSString()];
+            [m_popupButton.menu addItemWithTitle:filter.toNSString() action:nil keyEquivalent:@""];
         }
         if (filterToUse != -1)
             [m_popupButton selectItemAtIndex:filterToUse];
