@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qiosglobal.h"
 #include "qiosintegration.h"
@@ -213,12 +177,10 @@ static QIOSScreen* qtPlatformScreenFor(UIScreen *uiScreen)
 {
     [super traitCollectionDidChange:previousTraitCollection];
 
-    if (@available(iOS 12, *)) {
-        if (self.screen == UIScreen.mainScreen) {
-            if (previousTraitCollection.userInterfaceStyle != self.traitCollection.userInterfaceStyle) {
-                QIOSTheme::initializeSystemPalette();
-                QWindowSystemInterface::handleThemeChange<QWindowSystemInterface::SynchronousDelivery>(nullptr);
-            }
+    if (self.screen == UIScreen.mainScreen) {
+        if (previousTraitCollection.userInterfaceStyle != self.traitCollection.userInterfaceStyle) {
+            QIOSTheme::initializeSystemPalette();
+            QWindowSystemInterface::handleThemeChange<QWindowSystemInterface::SynchronousDelivery>();
         }
     }
 }
@@ -229,6 +191,8 @@ static QIOSScreen* qtPlatformScreenFor(UIScreen *uiScreen)
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 /*!
     Returns the model identifier of the device.
 */
@@ -290,7 +254,7 @@ QIOSScreen::QIOSScreen(UIScreen *screen)
     if (!qt_apple_isApplicationExtension()) {
         for (UIWindow *existingWindow in qt_apple_sharedApplication().windows) {
             if (existingWindow.screen == m_uiScreen) {
-                m_uiWindow = [m_uiWindow retain];
+                m_uiWindow = [existingWindow retain];
                 break;
             }
         }
@@ -321,12 +285,10 @@ QIOSScreen::~QIOSScreen()
 
 QString QIOSScreen::name() const
 {
-    if (m_uiScreen == [UIScreen mainScreen]) {
-        return QString::fromNSString([UIDevice currentDevice].model)
-            + QLatin1String(" built-in display");
-    } else {
-        return QLatin1String("External display");
-    }
+    if (m_uiScreen == [UIScreen mainScreen])
+        return QString::fromNSString([UIDevice currentDevice].model) + " built-in display"_L1;
+    else
+        return "External display"_L1;
 }
 
 void QIOSScreen::updateProperties()
@@ -566,6 +528,6 @@ UIWindow *QIOSScreen::uiWindow() const
     return m_uiWindow;
 }
 
-#include "moc_qiosscreen.cpp"
-
 QT_END_NAMESPACE
+
+#include "moc_qiosscreen.cpp"
