@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qxcbdrag.h"
 #include <xcb/xcb.h>
@@ -174,11 +138,9 @@ void QXcbDrag::startDrag()
 {
     init();
 
-#ifndef QT_NO_CLIPBOARD
-    qCDebug(lcQpaXDnd) << "starting drag where source:" << connection()->clipboard()->owner();
-    xcb_set_selection_owner(xcb_connection(), connection()->clipboard()->owner(),
+    qCDebug(lcQpaXDnd) << "starting drag where source:" << connection()->qtSelectionOwner();
+    xcb_set_selection_owner(xcb_connection(), connection()->qtSelectionOwner(),
                             atom(QXcbAtom::XdndSelection), connection()->time());
-#endif
 
     QStringList fmts = QXcbMime::formatsHelper(drag()->mimeData());
     for (int i = 0; i < fmts.size(); ++i) {
@@ -188,12 +150,11 @@ void QXcbDrag::startDrag()
                 drag_types.append(atoms.at(j));
         }
     }
-#ifndef QT_NO_CLIPBOARD
+
     if (drag_types.size() > 3)
-        xcb_change_property(xcb_connection(), XCB_PROP_MODE_REPLACE, connection()->clipboard()->owner(),
+        xcb_change_property(xcb_connection(), XCB_PROP_MODE_REPLACE, connection()->qtSelectionOwner(),
                             atom(QXcbAtom::XdndTypelist),
                             XCB_ATOM_ATOM, 32, drag_types.size(), (const void *)drag_types.constData());
-#endif
 
     setUseCompositing(current_virtual_desktop->compositingActive());
     setScreen(current_virtual_desktop->screens().constFirst()->screen());
@@ -444,11 +405,7 @@ void QXcbDrag::move(const QPoint &globalPos, Qt::MouseButtons b, Qt::KeyboardMod
             enter.window = target;
             enter.format = 32;
             enter.type = atom(QXcbAtom::XdndEnter);
-#ifndef QT_NO_CLIPBOARD
-            enter.data.data32[0] = connection()->clipboard()->owner();
-#else
-            enter.data.data32[0] = 0;
-#endif
+            enter.data.data32[0] = connection()->qtSelectionOwner();
             enter.data.data32[1] = flags;
             enter.data.data32[2] = drag_types.size() > 0 ? drag_types.at(0) : 0;
             enter.data.data32[3] = drag_types.size() > 1 ? drag_types.at(1) : 0;
@@ -479,11 +436,7 @@ void QXcbDrag::move(const QPoint &globalPos, Qt::MouseButtons b, Qt::KeyboardMod
         move.window = target;
         move.format = 32;
         move.type = atom(QXcbAtom::XdndPosition);
-#ifndef QT_NO_CLIPBOARD
-        move.data.data32[0] = connection()->clipboard()->owner();
-#else
-        move.data.data32[0] = 0;
-#endif
+        move.data.data32[0] = connection()->qtSelectionOwner();
         move.data.data32[1] = 0; // flags
         move.data.data32[2] = (globalPos.x() << 16) + globalPos.y();
         move.data.data32[3] = connection()->time();
@@ -529,11 +482,7 @@ void QXcbDrag::drop(const QPoint &globalPos, Qt::MouseButtons b, Qt::KeyboardMod
     drop.window = current_target;
     drop.format = 32;
     drop.type = atom(QXcbAtom::XdndDrop);
-#ifndef QT_NO_CLIPBOARD
-    drop.data.data32[0] = connection()->clipboard()->owner();
-#else
-    drop.data.data32[0] = 0;
-#endif
+    drop.data.data32[0] = connection()->qtSelectionOwner();
     drop.data.data32[1] = 0; // flags
     drop.data.data32[2] = connection()->time();
 
@@ -640,7 +589,7 @@ void QXcbDrag::setActionList(Qt::DropAction requestedAction, Qt::DropActions sup
     checkAppend(Qt::LinkAction);
 
     if (current_actions != actions) {
-        xcb_change_property(xcb_connection(), XCB_PROP_MODE_REPLACE, connection()->clipboard()->owner(),
+        xcb_change_property(xcb_connection(), XCB_PROP_MODE_REPLACE, connection()->qtSelectionOwner(),
                             atom(QXcbAtom::XdndActionList),
                             XCB_ATOM_ATOM, 32, actions.size(), actions.constData());
         current_actions = actions;
@@ -665,7 +614,7 @@ void QXcbDrag::stopListeningForActionListChanges()
 int QXcbDrag::findTransactionByWindow(xcb_window_t window)
 {
     int at = -1;
-    for (int i = 0; i < transactions.count(); ++i) {
+    for (int i = 0; i < transactions.size(); ++i) {
         const Transaction &t = transactions.at(i);
         if (t.target == window || t.proxy_target == window) {
             at = i;
@@ -678,7 +627,7 @@ int QXcbDrag::findTransactionByWindow(xcb_window_t window)
 int QXcbDrag::findTransactionByTime(xcb_timestamp_t timestamp)
 {
     int at = -1;
-    for (int i = 0; i < transactions.count(); ++i) {
+    for (int i = 0; i < transactions.size(); ++i) {
         const Transaction &t = transactions.at(i);
         if (t.timestamp == timestamp) {
             at = i;
@@ -771,7 +720,7 @@ void QXcbDrag::handleEnter(QPlatformWindow *, const xcb_client_message_event_t *
                 xdnd_types.append(event->data.data32[i]);
         }
     }
-    for(int i = 0; i < xdnd_types.length(); ++i)
+    for(int i = 0; i < xdnd_types.size(); ++i)
         qCDebug(lcQpaXDnd) << "    " << connection()->atomName(xdnd_types.at(i));
 }
 
@@ -785,7 +734,7 @@ void QXcbDrag::handle_xdnd_position(QPlatformWindow *w, const xcb_client_message
     QPoint p((e->data.data32[2] & 0xffff0000) >> 16, e->data.data32[2] & 0x0000ffff);
     Q_ASSERT(w);
     QRect geometry = w->geometry();
-    p -= geometry.topLeft();
+    p -= w->isEmbedded() ? w->mapToGlobal(geometry.topLeft()) : geometry.topLeft();
 
     if (!w || !w->window() || (w->window()->type() == Qt::Desktop))
         return;
@@ -861,11 +810,9 @@ void QXcbDrag::handle_xdnd_position(QPlatformWindow *w, const xcb_client_message
 
     qCDebug(lcQpaXDnd) << "sending XdndStatus to source:" << xdnd_dragsource;
 
-#ifndef QT_NO_CLIPBOARD
-    if (xdnd_dragsource == connection()->clipboard()->owner())
+    if (xdnd_dragsource == connection()->qtSelectionOwner())
         handle_xdnd_status(&response);
     else
-#endif
         xcb_send_event(xcb_connection(), false, current_proxy_target,
                        XCB_EVENT_MASK_NO_EVENT, (const char *)&response);
 }
@@ -931,11 +878,7 @@ void QXcbDrag::handle_xdnd_status(const xcb_client_message_event_t *event)
 
 void QXcbDrag::handleStatus(const xcb_client_message_event_t *event)
 {
-    if (
-#ifndef QT_NO_CLIPBOARD
-            event->window != connection()->clipboard()->owner() ||
-#endif
-            !drag())
+    if (event->window != connection()->qtSelectionOwner() || !drag())
         return;
 
     xcb_client_message_event_t *lastEvent = const_cast<xcb_client_message_event_t *>(event);
@@ -992,11 +935,7 @@ void QXcbDrag::send_leave()
     leave.window = current_target;
     leave.format = 32;
     leave.type = atom(QXcbAtom::XdndLeave);
-#ifndef QT_NO_CLIPBOARD
-    leave.data.data32[0] = connection()->clipboard()->owner();
-#else
-    leave.data.data32[0] = 0;
-#endif
+    leave.data.data32[0] = connection()->qtSelectionOwner();
     leave.data.data32[1] = 0; // flags
     leave.data.data32[2] = 0; // x, y
     leave.data.data32[3] = 0; // w, h
@@ -1041,21 +980,30 @@ void QXcbDrag::handleDrop(QPlatformWindow *, const xcb_client_message_event_t *e
 
     Qt::DropActions supported_drop_actions;
     QMimeData *dropData = nullptr;
+    // this could be a same-application drop, just proxied due to
+    // some XEMBEDding, so try to find the real QMimeData used
+    // based on the timestamp for this drop.
+    int at = findTransactionByTime(target_time);
+    if (at != -1) {
+        qCDebug(lcQpaXDnd) << "found one transaction via findTransactionByTime()";
+        dropData = transactions.at(at).drag->mimeData();
+        // Can't use the source QMimeData if we need the image conversion code from xdndObtainData
+        if (dropData && dropData->hasImage())
+            dropData = 0;
+    }
+    // if we can't find it, then use the data in the drag manager
     if (currentDrag()) {
-        dropData = currentDrag()->mimeData();
+        if (!dropData)
+            dropData = currentDrag()->mimeData();
         supported_drop_actions = Qt::DropActions(l[4]);
     } else {
-        dropData = m_dropData;
+        if (!dropData)
+            dropData = m_dropData;
         supported_drop_actions = accepted_drop_action | toDropActions(drop_actions);
     }
 
     if (!dropData)
         return;
-    // ###
-    //        int at = findXdndDropTransactionByTime(target_time);
-    //        if (at != -1)
-    //            dropData = QDragManager::dragPrivate(X11->dndDropTransactions.at(at).object)->data;
-    // if we can't find it, then use the data in the drag manager
 
     auto buttons = currentDrag() ? b : connection()->queryMouseButtons();
     auto modifiers = currentDrag() ? mods : connection()->queryKeyboardModifiers();
@@ -1064,7 +1012,12 @@ void QXcbDrag::handleDrop(QPlatformWindow *, const xcb_client_message_event_t *e
                 currentWindow.data(), dropData, currentPosition, supported_drop_actions,
                 buttons, modifiers);
 
-    setExecutedDropAction(response.acceptedAction());
+    Qt::DropAction acceptedAaction = response.acceptedAction();
+    if (!response.isAccepted()) {
+        // Ignore a failed drag
+        acceptedAaction = Qt::IgnoreAction;
+    }
+    setExecutedDropAction(acceptedAaction);
 
     xcb_client_message_event_t finished = {};
     finished.response_type = XCB_CLIENT_MESSAGE;
@@ -1074,7 +1027,7 @@ void QXcbDrag::handleDrop(QPlatformWindow *, const xcb_client_message_event_t *e
     finished.type = atom(QXcbAtom::XdndFinished);
     finished.data.data32[0] = currentWindow ? xcb_window(currentWindow.data()) : XCB_NONE;
     finished.data.data32[1] = response.isAccepted(); // flags
-    finished.data.data32[2] = toXdndAction(response.acceptedAction());
+    finished.data.data32[2] = toXdndAction(acceptedAaction);
 
     qCDebug(lcQpaXDnd) << "sending XdndFinished to source:" << xdnd_dragsource;
 
@@ -1091,10 +1044,8 @@ void QXcbDrag::handleFinished(const xcb_client_message_event_t *event)
     // Source receives XdndFinished when target is done processing the drop data.
     qCDebug(lcQpaXDnd) << "source:" << event->window << "received XdndFinished";
 
-#ifndef QT_NO_CLIPBOARD
-    if (event->window != connection()->clipboard()->owner())
+    if (event->window != connection()->qtSelectionOwner())
         return;
-#endif
 
     const unsigned long *l = (const unsigned long *)event->data.data32;
     if (l[0]) {
@@ -1138,7 +1089,7 @@ void QXcbDrag::timerEvent(QTimerEvent* e)
 {
     if (e->timerId() == cleanup_timer) {
         bool stopTimer = true;
-        for (int i = 0; i < transactions.count(); ++i) {
+        for (int i = 0; i < transactions.size(); ++i) {
             const Transaction &t = transactions.at(i);
             if (t.targetWindow) {
                 // dnd within the same process, don't delete, these are taken care of
@@ -1280,6 +1231,7 @@ void QXcbDrag::handleSelectionRequest(const xcb_selection_request_event_t *event
 
 bool QXcbDrag::dndEnable(QXcbWindow *w, bool on)
 {
+    qCDebug(lcQpaXDnd) << "dndEnable" << w << on;
     // Windows announce that they support the XDND protocol by creating a window property XdndAware.
     if (on) {
         QXcbWindow *window = nullptr;
@@ -1357,26 +1309,26 @@ QVariant QXcbDropData::xdndObtainData(const QByteArray &format, QMetaType reques
     QXcbWindow *xcb_window = c->platformWindowFromId(drag->xdnd_dragsource);
     if (xcb_window && drag->currentDrag() && xcb_window->window()->type() != Qt::Desktop) {
         QMimeData *data = drag->currentDrag()->mimeData();
-        if (data->hasFormat(QLatin1String(format)))
-            result = data->data(QLatin1String(format));
+        if (data->hasFormat(QLatin1StringView(format)))
+            result = data->data(QLatin1StringView(format));
         return result;
     }
 
     QList<xcb_atom_t> atoms = drag->xdnd_types;
     bool hasUtf8 = false;
-    xcb_atom_t a = mimeAtomForFormat(c, QLatin1String(format), requestedType, atoms, &hasUtf8);
+    xcb_atom_t a = mimeAtomForFormat(c, QLatin1StringView(format), requestedType, atoms, &hasUtf8);
     if (a == XCB_NONE)
         return result;
 
 #ifndef QT_NO_CLIPBOARD
-    if (c->clipboard()->getSelectionOwner(drag->atom(QXcbAtom::XdndSelection)) == XCB_NONE)
+    if (c->selectionOwner(c->atom(QXcbAtom::XdndSelection)) == XCB_NONE)
         return result; // should never happen?
 
     xcb_atom_t xdnd_selection = c->atom(QXcbAtom::XdndSelection);
     result = c->clipboard()->getSelection(xdnd_selection, a, xdnd_selection, drag->targetTime());
 #endif
 
-    return mimeConvertToFormat(c, a, result, QLatin1String(format), requestedType, hasUtf8);
+    return mimeConvertToFormat(c, a, result, QLatin1StringView(format), requestedType, hasUtf8);
 }
 
 bool QXcbDropData::hasFormat_sys(const QString &format) const
