@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <qplatformdefs.h>
 #include <private/qabstractspinbox_p.h>
@@ -58,7 +22,7 @@
 #include <qpalette.h>
 #include <qstylepainter.h>
 #include <qdebug.h>
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 # include <qaccessible.h>
 #endif
 
@@ -72,6 +36,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 /*!
     \class QAbstractSpinBox
     \brief The QAbstractSpinBox class provides a spinbox and a line edit to
@@ -671,7 +637,8 @@ void QAbstractSpinBox::stepBy(int steps)
     } else if (e == AlwaysEmit) {
         d->emitSignals(e, old);
     }
-    selectAll();
+    if (style()->styleHint(QStyle::SH_SpinBox_SelectOnStep, nullptr, this, nullptr))
+        selectAll();
 }
 
 /*!
@@ -905,7 +872,7 @@ QSize QAbstractSpinBox::sizeHint() const
         int h = d->edit->sizeHint().height();
         int w = 0;
         QString s;
-        QString fixedContent =  d->prefix + d->suffix + QLatin1Char(' ');
+        QString fixedContent =  d->prefix + d->suffix + u' ';
         s = d->textFromValue(d->minimum);
         s.truncate(18);
         s += fixedContent;
@@ -945,7 +912,7 @@ QSize QAbstractSpinBox::minimumSizeHint() const
         int w = 0;
 
         QString s;
-        QString fixedContent =  d->prefix + QLatin1Char(' ');
+        QString fixedContent =  d->prefix + u' ';
         s = d->textFromValue(d->minimum);
         s.truncate(18);
         s += fixedContent;
@@ -1053,7 +1020,7 @@ void QAbstractSpinBox::keyPressEvent(QKeyEvent *event)
                 d->updateState(up, true);
             }
         }
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         QAccessibleValueChangeEvent event(this, d->value);
         QAccessible::updateAccessibility(&event);
 #endif
@@ -1095,7 +1062,7 @@ void QAbstractSpinBox::keyPressEvent(QKeyEvent *event)
 
     case Qt::Key_U:
         if (event->modifiers() & Qt::ControlModifier
-            && QGuiApplication::platformName() == QLatin1String("xcb")) { // only X11
+            && QGuiApplication::platformName() == "xcb"_L1) { // only X11
             event->accept();
             if (!isReadOnly())
                 clear();
@@ -1624,7 +1591,7 @@ void QAbstractSpinBoxPrivate::init()
     Q_Q(QAbstractSpinBox);
 
     q->setLineEdit(new QLineEdit(q));
-    edit->setObjectName(QLatin1String("qt_spinbox_lineedit"));
+    edit->setObjectName("qt_spinbox_lineedit"_L1);
     validator = new QSpinBoxValidator(q, this);
     edit->setValidator(validator);
 
@@ -1684,7 +1651,7 @@ void QAbstractSpinBoxPrivate::updateState(bool up, bool fromKeyboard /* = false
             steps *= 10;
         q->stepBy(steps);
         spinClickThresholdTimerId = q->startTimer(spinClickThresholdTimerInterval);
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
         QAccessibleValueChangeEvent event(q, value);
         QAccessible::updateAccessibility(&event);
 #endif
@@ -1968,7 +1935,7 @@ QVariant QAbstractSpinBoxPrivate::calculateAdaptiveDecimalStep(int steps) const
 QSpinBoxValidator::QSpinBoxValidator(QAbstractSpinBox *qp, QAbstractSpinBoxPrivate *dp)
     : QValidator(qp), qptr(qp), dptr(dp)
 {
-    setObjectName(QLatin1String("qt_spinboxvalidator"));
+    setObjectName("qt_spinboxvalidator"_L1);
 }
 
 /*!
@@ -1985,7 +1952,7 @@ QValidator::State QSpinBoxValidator::validate(QString &input, int &pos) const
 
     if (!dptr->prefix.isEmpty() && !input.startsWith(dptr->prefix)) {
         input.prepend(dptr->prefix);
-        pos += dptr->prefix.length();
+        pos += dptr->prefix.size();
     }
 
     if (!dptr->suffix.isEmpty() && !input.endsWith(dptr->suffix))
