@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qtoolbar.h"
 
@@ -736,117 +700,6 @@ void QToolBar::clear()
         removeAction(actions.at(i));
 }
 
-/*!
-    Creates a new action with the given \a text. This action is added to
-    the end of the toolbar.
-*/
-QAction *QToolBar::addAction(const QString &text)
-{
-    QAction *action = new QAction(text, this);
-    addAction(action);
-    return action;
-}
-
-/*!
-    \overload
-
-    Creates a new action with the given \a icon and \a text. This
-    action is added to the end of the toolbar.
-*/
-QAction *QToolBar::addAction(const QIcon &icon, const QString &text)
-{
-    QAction *action = new QAction(icon, text, this);
-    addAction(action);
-    return action;
-}
-
-/*!
-    \overload
-
-    Creates a new action with the given \a text. This action is added to
-    the end of the toolbar. The action's \l{QAction::triggered()}{triggered()}
-    signal is connected to \a member in \a receiver.
-*/
-QAction *QToolBar::addAction(const QString &text,
-                             const QObject *receiver, const char* member)
-{
-    QAction *action = new QAction(text, this);
-    QObject::connect(action, SIGNAL(triggered(bool)), receiver, member);
-    addAction(action);
-    return action;
-}
-
-/*!
-    \overload
-
-    Creates a new action with the given \a icon and \a text. This
-    action is added to the end of the toolbar. The action's
-    \l{QAction::triggered()}{triggered()} signal is connected to \a
-    member in \a receiver.
-*/
-QAction *QToolBar::addAction(const QIcon &icon, const QString &text,
-                             const QObject *receiver, const char* member)
-{
-    QAction *action = new QAction(icon, text, this);
-    QObject::connect(action, SIGNAL(triggered(bool)), receiver, member);
-    addAction(action);
-    return action;
-}
-
-/*!\fn template<typename Functor> QAction *QToolBar::addAction(const QString &text, Functor functor)
-
-    \since 5.6
-
-    \overload
-
-    Creates a new action with the given \a text. This action is added to
-    the end of the toolbar. The action's
-    \l{QAction::triggered()}{triggered()} signal is connected to the
-    \a functor.
-*/
-
-/*!\fn template<typename Functor> QAction *QToolBar::addAction(const QString &text, const QObject *context, Functor functor)
-
-    \since 5.6
-
-    \overload
-
-    Creates a new action with the given \a text. This action is added
-    to the end of the toolbar. The action's
-    \l{QAction::triggered()}{triggered()} signal is connected to the
-    \a functor. The \a functor can be a pointer to a member function
-    in the \a context object.
-
-    If the \a context object is destroyed, the \a functor will not be called.
-*/
-
-/*!\fn template<typename Functor> QAction *QToolBar::addAction(const QIcon &icon, const QString &text, Functor functor)
-
-    \since 5.6
-
-    \overload
-
-    Creates a new action with the given \a icon and \a text. This
-    action is added to the end of the toolbar. The action's
-    \l{QAction::triggered()}{triggered()} signal is connected to the
-    \a functor.
-*/
-
-/*!\fn template<typename Functor> QAction *QToolBar::addAction(const QIcon &icon, const QString &text, const QObject *context, Functor functor)
-
-    \since 5.6
-
-    \overload
-
-    Creates a new action with the given \a icon and \a text. This
-    action is added to the end of the toolbar. The action's
-    \l{QAction::triggered()}{triggered()} signal is connected to the
-    \a functor. The \a functor can be a pointer to a member function
-    of the \a context object.
-
-    If the \a context object is destroyed, the \a functor will not be called.
-*/
-
 /*!
      Adds a separator to the end of the toolbar.
 
