@@ -1,33 +1,8 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "tst_qmetatype.h"
-#include "tst_qvariant_common.h"
+#include "tst_qmetatype_libs.h"
 
 #include <QtCore/private/qmetaobjectbuilder_p.h>
 
@@ -164,6 +139,7 @@ void tst_QMetaType::convertCustomType_data()
     QTest::addColumn<QLineF>("testQLineF");
     QTest::addColumn<QChar>("testQChar");
     QTest::addColumn<CustomConvertibleType>("testCustom");
+    QTest::addColumn<DerivedGadgetType>("testDerived");
 
     QTest::newRow("default") << true
                              << QString::fromLatin1("string") << true << 15
@@ -171,14 +147,16 @@ void tst_QMetaType::convertCustomType_data()
                              << QRectF(1.4, 1.9, 10.9, 40.2) << QPoint(12, 34)
                              << QPointF(9.2, 2.7) << QSize(4, 9) << QSizeF(3.3, 9.8)
                              << QLine(3, 9, 29, 4) << QLineF(38.9, 28.9, 102.3, 0.0)
-                             << QChar('Q') << CustomConvertibleType(QString::fromLatin1("test"));
+                             << QChar('Q') << CustomConvertibleType(QString::fromLatin1("test"))
+                             << DerivedGadgetType(QString::fromLatin1("test"));
     QTest::newRow("not ok") << false
                             << QString::fromLatin1("string") << true << 15
                             << double(3.14) << float(3.6) << QRect(1, 2, 3, 4)
                             << QRectF(1.4, 1.9, 10.9, 40.2) << QPoint(12, 34)
                             << QPointF(9.2, 2.7) << QSize(4, 9) << QSizeF(3.3, 9.8)
                             << QLine(3, 9, 29, 4) << QLineF()
-                            << QChar('Q') << CustomConvertibleType(42);
+                            << QChar('Q') << CustomConvertibleType(42)
+                            << DerivedGadgetType(42);
 }
 
 void tst_QMetaType::convertCustomType()
@@ -276,6 +254,15 @@ void tst_QMetaType::convertCustomType()
     v = QVariant::fromValue(testCustom);
     QVERIFY(v.canConvert(::qMetaTypeId<CustomConvertibleType2>()));
     QCOMPARE(v.value<CustomConvertibleType2>().m_foo, testCustom.m_foo);
+
+    QFETCH(DerivedGadgetType, testDerived);
+    v = QVariant::fromValue(testDerived);
+    QCOMPARE(v.metaType(), QMetaType::fromType<DerivedGadgetType>());
+    QCOMPARE(v.value<DerivedGadgetType>().m_foo, testDerived.m_foo);
+    QVERIFY(v.canConvert(QMetaType::fromType<BaseGadgetType>()));
+    QVERIFY(v.convert(QMetaType::fromType<BaseGadgetType>()));
+    QCOMPARE(v.metaType(), QMetaType::fromType<BaseGadgetType>());
+    QCOMPARE(v.value<BaseGadgetType>().m_foo, testDerived.m_foo);
 }
 
 void tst_QMetaType::convertConstNonConst()
@@ -425,6 +412,124 @@ void tst_QMetaType::operatorEq()
     QCOMPARE(typeB == typeA, eq);
     QCOMPARE(typeA != typeB, !eq);
     QCOMPARE(typeB != typeA, !eq);
+
+#if !defined(Q_OS_WIN) && !defined(Q_OS_INTEGRITY)
+    // for built-in types or locally-defined types, this must also hold true
+    if (eq)
+        QCOMPARE(typeA.iface(), typeB.iface());
+#endif
+}
+
+void tst_QMetaType::operatorEq2_data()
+{
+    create_data();
+}
+
+void tst_QMetaType::operatorEq2()
+{
+    QFETCH(int, type);
+    QMetaType fromType1, fromType2;
+    QMetaType fromId1(type), fromId2(type);
+
+    switch (type) {
+    case QMetaType::UnknownType:
+        break;
+#define GET_METATYPE_FROM_TYPE(MetaTypeName, MetaTypeId, RealType)       \
+    case QMetaType::MetaTypeName:                           \
+        fromType1 = QMetaType::fromType<RealType>();        \
+        fromType2 = QMetaType::fromType<RealType>();        \
+        break;
+FOR_EACH_CORE_METATYPE(GET_METATYPE_FROM_TYPE)
+#undef GET_METATYPE_FROM_TYPE
+    }
+
+    // sanity check
+    QCOMPARE(fromId1.id(), type);
+    QCOMPARE(fromId2.id(), type);
+
+    // confirm that they're all equal
+    QCOMPARE(fromId1, fromId2);
+    QCOMPARE(fromType1, fromType2);
+    QCOMPARE(fromType1, fromId1);
+    QCOMPARE(fromType2, fromId2);
+
+#if !defined(Q_OS_WIN) && !defined(Q_OS_INTEGRITY)
+    // for built-in types (other than void), this must be true
+    if (type != QMetaType::Void) {
+        QCOMPARE(fromType1.iface(), fromId1.iface());
+        QCOMPARE(fromType2.iface(), fromId1.iface());
+    }
+#endif
+}
+
+#define DECLARE_LIB_FUNCTION(TYPE, ID)              \
+    Q_DECL_IMPORT QMetaType metatype_ ## TYPE();
+namespace Lib1 { FOR_EACH_METATYPE_LIBS(DECLARE_LIB_FUNCTION) }
+namespace Lib2 { FOR_EACH_METATYPE_LIBS(DECLARE_LIB_FUNCTION) }
+#undef DECLARE_LIB_FUNCTION
+
+using LibMetatypeFunction = QMetaType (*)();
+void tst_QMetaType::operatorEqAcrossLibs_data()
+{
+    QTest::addColumn<int>("builtinTypeId");
+    QTest::addColumn<QMetaType>("localType");
+    QTest::addColumn<LibMetatypeFunction>("lib1Function");
+    QTest::addColumn<LibMetatypeFunction>("lib2Function");
+
+#define ADD_ROW(TYPE, ID)                           \
+    QTest::addRow(QT_STRINGIFY(TYPE))   << int(ID)  \
+        << QMetaType::fromType<TYPE>()              \
+        << &Lib1::metatype_ ## TYPE                 \
+        << &Lib2::metatype_ ## TYPE;
+FOR_EACH_METATYPE_LIBS(ADD_ROW)
+#undef ADD_ROW
+}
+
+void tst_QMetaType::operatorEqAcrossLibs()
+{
+    QFETCH(int, builtinTypeId);
+    QFETCH(QMetaType, localType);
+    QFETCH(LibMetatypeFunction, lib1Function);
+    QFETCH(LibMetatypeFunction, lib2Function);
+
+    QMetaType lib1Type = lib1Function();
+    QMetaType lib2Type = lib2Function();
+
+    const QtPrivate::QMetaTypeInterface *localIface = localType.iface();
+    const QtPrivate::QMetaTypeInterface *lib1Iface = lib1Type.iface();
+    const QtPrivate::QMetaTypeInterface *lib2Iface = lib2Type.iface();
+
+    // DO THIS FIRST:
+    // if this isn't a built-in type, then the QMetaTypeInterface::typeId is
+    // initially set to 0
+    QCOMPARE(lib1Type, lib2Type);
+
+    int actualTypeId = localType.id();
+    bool builtinTypeExpected = builtinTypeId != QMetaType::UnknownType;
+    bool builtinTypeActually = actualTypeId < QMetaType::User;
+
+    qDebug() << "QMetaType for type" << QByteArray(localType.name())
+             << "(type ID" << (actualTypeId >= 0x1000 ? Qt::hex : Qt::dec) << actualTypeId << ')'
+             << (builtinTypeActually ? "IS" : "is NOT") << "a built-in type;"
+             << "local interface:" << static_cast<const void *>(localIface)
+             << "lib1 interface:" << static_cast<const void *>(lib1Iface)
+             << "lib2 interface:" << static_cast<const void *>(lib2Iface);
+
+    QCOMPARE(builtinTypeActually, builtinTypeExpected);
+    QCOMPARE(lib1Type.id(), actualTypeId);
+    QCOMPARE(lib2Type.id(), actualTypeId);
+    QCOMPARE(QByteArray(lib1Type.name()), QByteArray(localType.name()));
+    QCOMPARE(QByteArray(lib2Type.name()), QByteArray(localType.name()));
+    QCOMPARE(lib1Type, localType);
+    QCOMPARE(lib2Type, localType);
+
+#if !defined(Q_OS_WIN) && !defined(Q_OS_INTEGRITY)
+    if (actualTypeId < QMetaType::FirstGuiType && actualTypeId != QMetaType::Void) {
+        // for built-in QtCore types, we expect the interfaces to be the same too
+        QCOMPARE(lib1Iface, localIface);
+        QCOMPARE(lib2Iface, localIface);
+    }
+#endif
 }
 
 class WithPrivateDTor {
