@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <qmimedatabase.h>
 
@@ -78,12 +53,12 @@ static inline QString testSuiteWarning()
     str << "\nCannot find the shared-mime-info test suite\nin the parent of: "
         << QDir::toNativeSeparators(QDir::currentPath()) << "\n"
            "cd " << QDir::toNativeSeparators(QStringLiteral("tests/auto/corelib/mimetypes/qmimedatabase")) << "\n"
-           "wget https://gitlab.freedesktop.org/xdg/shared-mime-info/-/archive/2.1/shared-mime-info-2.1.zip\n"
-           "unzip shared-mime-info-2.1.zip\n";
+           "wget https://gitlab.freedesktop.org/xdg/shared-mime-info/-/archive/2.2/shared-mime-info-2.2.zip\n"
+           "unzip shared-mime-info-2.2.zip\n";
 #ifdef Q_OS_WIN
-    str << "mkdir testfiles\nxcopy /s shared-mime-info-2.1 s-m-i\n";
+    str << "mkdir testfiles\nxcopy /s shared-mime-info-2.2 s-m-i\n";
 #else
-    str << "ln -s shared-mime-info-2.1 s-m-i\n";
+    str << "ln -s shared-mime-info-2.2 s-m-i\n";
 #endif
     return result;
 }
@@ -302,7 +277,7 @@ void tst_QMimeDatabase::mimeTypeForFileName()
         QVERIFY(mimes.isEmpty());
     } else {
         QVERIFY2(!mimes.isEmpty(), msgMimeTypeForFileNameFailed(mimes, expectedMimeType).constData());
-        QVERIFY2(mimes.count() == 1, msgMimeTypeForFileNameFailed(mimes, expectedMimeType).constData());
+        QVERIFY2(mimes.size() == 1, msgMimeTypeForFileNameFailed(mimes, expectedMimeType).constData());
         QCOMPARE(mimes.first().name(), expectedMimeType);
     }
 }
@@ -353,7 +328,7 @@ void tst_QMimeDatabase::inheritance()
 
     const QMimeType directory = db.mimeTypeForName(QString::fromLatin1("inode/directory"));
     QVERIFY(directory.isValid());
-    QCOMPARE(directory.parentMimeTypes().count(), 0);
+    QCOMPARE(directory.parentMimeTypes().size(), 0);
     QVERIFY(!directory.inherits(QLatin1String("application/octet-stream")));
 
     // Check that text/x-patch knows that it inherits from text/plain (it says so explicitly)
@@ -374,7 +349,7 @@ void tst_QMimeDatabase::inheritance()
     const QStringList shellParents = shellscript.parentMimeTypes();
     QVERIFY(shellParents.contains(QLatin1String("text/plain")));
     QVERIFY(shellParents.contains(QLatin1String("application/x-executable")));
-    QCOMPARE(shellParents.count(), 2); // only the above two
+    QCOMPARE(shellParents.size(), 2); // only the above two
     const QStringList allShellAncestors = shellscript.allAncestors();
     QVERIFY(allShellAncestors.contains(QLatin1String("text/plain")));
     QVERIFY(allShellAncestors.contains(QLatin1String("application/x-executable")));
@@ -616,7 +591,7 @@ void tst_QMimeDatabase::allMimeTypes()
     QVERIFY(!lst.isEmpty());
 
     // Hardcoding this is the only way to check both providers find the same number of mimetypes.
-    QCOMPARE(lst.count(), 811);
+    QCOMPARE(lst.size(), 851);
 
     foreach (const QMimeType &mime, lst) {
         const QString name = mime.name();
@@ -766,7 +741,7 @@ void tst_QMimeDatabase::findByFileName()
     //qDebug() << Q_FUNC_INFO << "mimeTypeForFile() returned" << resultMimeTypeName;
 
     const bool failed = resultMimeTypeName != mimeTypeName;
-    const bool shouldFail = (xFail.length() >= 1 && xFail.at(0) == QLatin1Char('x'));
+    const bool shouldFail = (xFail.size() >= 1 && xFail.at(0) == QLatin1Char('x'));
     if (shouldFail != failed) {
         // Results are ambiguous when multiple MIME types have the same glob
         // -> accept the current result if the found MIME type actually
@@ -813,7 +788,7 @@ void tst_QMimeDatabase::findByData()
     QByteArray data = f.read(16384);
 
     const QString resultMimeTypeName = database.mimeTypeForData(data).name();
-    if (xFail.length() >= 2 && xFail.at(1) == QLatin1Char('x')) {
+    if (xFail.size() >= 2 && xFail.at(1) == QLatin1Char('x')) {
         // Expected to fail
         QVERIFY2(resultMimeTypeName != mimeTypeName, qPrintable(resultMimeTypeName));
     } else {
@@ -844,7 +819,7 @@ void tst_QMimeDatabase::findByFile()
     QMimeDatabase database;
     const QString resultMimeTypeName = database.mimeTypeForFile(filePath).name();
     //qDebug() << Q_FUNC_INFO << filePath << "->" << resultMimeTypeName;
-    if (xFail.length() >= 3 && xFail.at(2) == QLatin1Char('x')) {
+    if (xFail.size() >= 3 && xFail.at(2) == QLatin1Char('x')) {
         // Expected to fail
         QVERIFY2(resultMimeTypeName != mimeTypeName, qPrintable(resultMimeTypeName));
     } else {
