@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 
 #include <QTest>
@@ -104,11 +79,9 @@ enum QLibraryOperation {
     QString sys_qualifiedLibraryName(const QString &fileName);
 
     QString directory;
-#ifdef Q_OS_ANDROID
-    QSharedPointer<QTemporaryDir> temporaryDir;
-#endif
 private slots:
     void initTestCase();
+    void cleanup();
 
     void load();
     void load_data();
@@ -141,24 +114,9 @@ typedef int (*VersionFunction)(void);
 void tst_QLibrary::initTestCase()
 {
 #ifdef Q_OS_ANDROID
-    auto tempDir = QEXTRACTTESTDATA("android_test_data");
-
-    QVERIFY2(QDir::setCurrent(tempDir->path()), qPrintable("Could not chdir to " + tempDir->path()));
-
-    // copy :/library_path into ./library_path
-    QVERIFY(QDir().mkdir("library_path"));
-    QDirIterator iterator(":/library_path", QDirIterator::Subdirectories);
-    while (iterator.hasNext()) {
-        iterator.next();
-        QFileInfo sourceFileInfo(iterator.path());
-        QFileInfo targetFileInfo("./library_path/" + sourceFileInfo.fileName());
-        if (!targetFileInfo.exists()) {
-            QDir().mkpath(targetFileInfo.path());
-            QVERIFY(QFile::copy(sourceFileInfo.filePath(), targetFileInfo.filePath()));
-        }
-    }
-    directory = tempDir->path();
-    temporaryDir = std::move(tempDir);
+    const QStringList paths = QCoreApplication::libraryPaths();
+    QVERIFY(!paths.isEmpty());
+    directory = paths.first();
 #else
     // chdir to our testdata directory, and use relative paths in some tests.
     QString testdatadir = QFileInfo(QFINDTESTDATA("library_path")).absolutePath();
@@ -167,6 +125,38 @@ void tst_QLibrary::initTestCase()
 #endif
 }
 
+void tst_QLibrary::cleanup()
+{
+    // unload the libraries, if they are still loaded after the test ended
+    // (probably in a failure)
+
+    static struct {
+        QString name;
+        int version = -1;
+    } libs[] = {
+        { directory + "/mylib" },
+        { directory + "/mylib", 1 },
+        { directory + "/mylib", 2 },
+        { sys_qualifiedLibraryName("mylib") },
+
+        // stuff that load_data() succeeds with
+        { directory + "/" PREFIX "mylib" },
+        { directory + "/" PREFIX "mylib" SUFFIX },
+#if defined(Q_OS_WIN32)
+        { directory + "/mylib.dl2" },
+        { directory + "/system.qt.test.mylib.dll" },
+#elif !defined(Q_OS_ANDROID)
+        // .so even on macOS
+        { directory + "/libmylib.so2" },
+        { directory + "/system.qt.test.mylib.so" },
+#endif
+
+    };
+    for (const auto &entry : libs) {
+        do {} while (QLibrary(entry.name, entry.version).unload());
+    }
+}
+
 void tst_QLibrary::version_data()
 {
 #ifdef Q_OS_ANDROID
@@ -224,7 +214,10 @@ void tst_QLibrary::load_data()
     QTest::newRow( "ok03 (with many dots)" ) << appDir + "/system.qt.test.mylib.dll" << true;
 # elif defined Q_OS_UNIX
     QTest::newRow( "ok01 (with suffix)" ) << appDir + "/libmylib" SUFFIX << true;
+#ifndef Q_OS_ANDROID
+    // We do not support non-standard suffixes on Android
     QTest::newRow( "ok02 (with non-standard suffix)" ) << appDir + "/libmylib.so2" << true;
+#endif
     QTest::newRow( "ok03 (with many dots)" ) << appDir + "/system.qt.test.mylib.so" << true;
 # endif  // Q_OS_UNIX
 }
@@ -438,7 +431,10 @@ void tst_QLibrary::loadHints_data()
     QTest::newRow( "ok03 (with many dots)" ) << appDir + "/system.qt.test.mylib.dll" << int(lh) << true;
 # elif defined Q_OS_UNIX
     QTest::newRow( "ok01 (with suffix)" ) << appDir + "/libmylib" SUFFIX << int(lh) << true;
+#ifndef Q_OS_ANDROID
+    // We do not support non-standard suffixes on Android
     QTest::newRow( "ok02 (with non-standard suffix)" ) << appDir + "/libmylib.so2" << int(lh) << true;
+#endif
     QTest::newRow( "ok03 (with many dots)" ) << appDir + "/system.qt.test.mylib.so" << int(lh) << true;
 # endif  // Q_OS_UNIX
 }
