@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 #include <QCoreApplication>
 #include <QDebug>
 
@@ -64,6 +39,8 @@ private slots:
     void cancelWhenDestroyed();
 #endif
     void cancelWhenReassigned();
+    void cancelWhenDestroyedWithoutStarting();
+    void cancelWhenDestroyedRunsContinuations();
     void finishWhenSwapped();
     void cancelWhenMoved();
     void waitUntilResumed();
@@ -505,6 +482,38 @@ void tst_QPromise::cancelWhenReassigned()
 #endif
 }
 
+void tst_QPromise::cancelWhenDestroyedWithoutStarting()
+{
+    QFuture<void> future;
+    {
+        QPromise<void> promise;
+        future = promise.future();
+    }
+    future.waitForFinished();
+    QVERIFY(!future.isStarted());
+    QVERIFY(future.isCanceled());
+    QVERIFY(future.isFinished());
+}
+
+void tst_QPromise::cancelWhenDestroyedRunsContinuations()
+{
+    QFuture<void> future;
+    bool onCanceledCalled = false;
+    bool thenCalled = false;
+    {
+        QPromise<void> promise;
+        future = promise.future();
+        future.then([&] {
+            thenCalled = true;
+        }).onCanceled([&] {
+            onCanceledCalled = true;
+        });
+    }
+    QVERIFY(future.isFinished());
+    QVERIFY(!thenCalled);
+    QVERIFY(onCanceledCalled);
+}
+
 void tst_QPromise::finishWhenSwapped()
 {
 #if !QT_CONFIG(cxx11_future)
