@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <qmap.h>
 #include <QTest>
@@ -447,7 +422,12 @@ void tst_QMap::beginEnd()
     // detach
     map2.insert( "2", "c" );
     QVERIFY( map.constBegin() == map.constBegin() );
-    QVERIFY( map.constBegin() != map2.constBegin() );
+
+    // comparing iterators between two different std::map is UB (and raises an
+    // assertion failure with MSVC debug-mode iterators), so we compare the
+    // elements' addresses.
+    QVERIFY(&map.constBegin().key() != &map2.constBegin().key());
+    QVERIFY(&map.constBegin().value() != &map2.constBegin().value());
 }
 
 void tst_QMap::firstLast()
@@ -1861,12 +1841,6 @@ void tst_QMap::equal_range()
     QCOMPARE(map.count(1), 2);
 }
 
-template <class T>
-const T &const_(const T &t)
-{
-    return t;
-}
-
 void tst_QMap::insert()
 {
     QMap<QString, float> map;
