@@ -1,33 +1,8 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QtTest/QTest>
-#include <qvarlengtharray.h>
+#include <QVarLengthArray>
 #include <qvariant.h>
 #include <qscopeguard.h>
 #include <qscopedvaluerollback.h>
@@ -72,8 +47,10 @@ private slots:
     void defaultConstructor_int() { defaultConstructor<int>(); }
     void defaultConstructor_QString() { defaultConstructor<QString>(); }
     void append();
+#if QT_DEPRECATED_SINCE(6, 3)
     void prepend();
-    void insertToEmpty();
+#endif
+    void emplace();
     void move_int_1() { move_int<1>(); }
     void move_int_2() { move_int<2>(); }
     void move_int_3() { move_int<3>(); }
@@ -169,6 +146,9 @@ void tst_QVarLengthArray::append()
     v2.append(5);
 }
 
+#if QT_DEPRECATED_SINCE(6, 3)
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_DEPRECATED
 void tst_QVarLengthArray::prepend()
 {
     QVarLengthArray<QString, 2> v;
@@ -187,35 +167,52 @@ void tst_QVarLengthArray::prepend()
     v.prepend(v.back());
     QCOMPARE(v.front(), v.back());
 }
+QT_WARNING_POP
+#endif // QT_DEPRECATED_SINCE(6, 3)
 
-void tst_QVarLengthArray::insertToEmpty()
+void tst_QVarLengthArray::emplace()
 {
-    // test growing from empty arrays
-    QVarLengthArray<QString> emptyArrDefaultPrealloc;
-    QCOMPARE(emptyArrDefaultPrealloc.size(), 0);
-    emptyArrDefaultPrealloc.insert(emptyArrDefaultPrealloc.cend(), QString());
-    QCOMPARE(emptyArrDefaultPrealloc.size(), 1);
-    emptyArrDefaultPrealloc.resize(1024);
-    QCOMPARE(emptyArrDefaultPrealloc.size(), 1024);
-    emptyArrDefaultPrealloc.resize(0);
-    QCOMPARE(emptyArrDefaultPrealloc.size(), 0);
-    emptyArrDefaultPrealloc.squeeze();
-    QCOMPARE(emptyArrDefaultPrealloc.size(), 0);
-    emptyArrDefaultPrealloc.insert(emptyArrDefaultPrealloc.cend(), QString());
-    QCOMPARE(emptyArrDefaultPrealloc.size(), 1);
+    {
+        QVarLengthArray<QString, 2> strings;
+        strings.emplace_back();
+        QCOMPARE(strings.size(), 1);
+        QCOMPARE(strings.front().isNull(), true);
+        strings.emplace(strings.begin(), 42, u'x');
+        QCOMPARE(strings.size(), 2);
+        QCOMPARE(strings.back().isNull(), true);
+        QCOMPARE(strings.front(), QString(42, u'x'));
+        auto &r = strings.emplace_back(42, u'y');
+        QCOMPARE(&r, &strings.back());
+        QCOMPARE(strings.size(), 3);
+        QCOMPARE(strings.back(), QString(42, u'y'));
 
-    QVarLengthArray<QString, 1> emptyArrSmallPrealloc;
-    QCOMPARE(emptyArrSmallPrealloc.size(), 0);
-    emptyArrSmallPrealloc.insert(emptyArrSmallPrealloc.cend(), QString());
-    QCOMPARE(emptyArrSmallPrealloc.size(), 1);
-    emptyArrSmallPrealloc.resize(1024);
-    QCOMPARE(emptyArrSmallPrealloc.size(), 1024);
-    emptyArrSmallPrealloc.resize(0);
-    QCOMPARE(emptyArrSmallPrealloc.size(), 0);
-    emptyArrSmallPrealloc.squeeze();
-    QCOMPARE(emptyArrSmallPrealloc.size(), 0);
-    emptyArrSmallPrealloc.insert(emptyArrSmallPrealloc.cend(), QString());
-    QCOMPARE(emptyArrSmallPrealloc.size(), 1);
+        // test growing from empty arrays
+        QVarLengthArray<QString> emptyArrDefaultPrealloc;
+        QCOMPARE(emptyArrDefaultPrealloc.size(), 0);
+        emptyArrDefaultPrealloc.emplace_back();
+        QCOMPARE(emptyArrDefaultPrealloc.size(), 1);
+        emptyArrDefaultPrealloc.resize(1024);
+        QCOMPARE(emptyArrDefaultPrealloc.size(), 1024);
+        emptyArrDefaultPrealloc.resize(0);
+        QCOMPARE(emptyArrDefaultPrealloc.size(), 0);
+        emptyArrDefaultPrealloc.squeeze();
+        QCOMPARE(emptyArrDefaultPrealloc.size(), 0);
+        emptyArrDefaultPrealloc.emplace_back();
+        QCOMPARE(emptyArrDefaultPrealloc.size(), 1);
+
+        QVarLengthArray<QString, 1> emptyArrSmallPrealloc;
+        QCOMPARE(emptyArrSmallPrealloc.size(), 0);
+        emptyArrSmallPrealloc.emplace_back();
+        QCOMPARE(emptyArrSmallPrealloc.size(), 1);
+        emptyArrSmallPrealloc.resize(1024);
+        QCOMPARE(emptyArrSmallPrealloc.size(), 1024);
+        emptyArrSmallPrealloc.resize(0);
+        QCOMPARE(emptyArrSmallPrealloc.size(), 0);
+        emptyArrSmallPrealloc.squeeze();
+        QCOMPARE(emptyArrSmallPrealloc.size(), 0);
+        emptyArrSmallPrealloc.emplace_back();
+        QCOMPARE(emptyArrSmallPrealloc.size(), 1);
+    }
 }
 
 template <qsizetype N>
@@ -414,20 +411,40 @@ void tst_QVarLengthArray::appendIsStronglyExceptionSafe()
     };
 
     {
-        // ### TODO: QVLA isn't exception-safe when throwing during reallocation,
-        // ### so check with size() < capacity() for now
         QVarLengthArray<Thrower, 2> vla(1);
         {
             Thrower t;
             const QScopedValueRollback rb(throwOnCopyNow, true);
-            QVERIFY_EXCEPTION_THROWN(vla.push_back(t), int);
+            QVERIFY_THROWS_EXCEPTION(int, vla.push_back(t));
             QCOMPARE(vla.size(), 1);
         }
         {
             const QScopedValueRollback rb(throwOnMoveNow, true);
-            QVERIFY_EXCEPTION_THROWN(vla.push_back({}), int);
+            QVERIFY_THROWS_EXCEPTION(int, vla.push_back({}));
             QCOMPARE(vla.size(), 1);
         }
+        vla.push_back({});
+        QCOMPARE(vla.size(), 2);
+        {
+            Thrower t;
+            {
+                // tests the copy inside append()
+                const QScopedValueRollback rb(throwOnCopyNow, true);
+                QVERIFY_THROWS_EXCEPTION(int, vla.push_back(t));
+                QCOMPARE(vla.size(), 2);
+            }
+            {
+                // tests the move inside reallocate()
+                const QScopedValueRollback rb(throwOnMoveNow, true);
+                QVERIFY_THROWS_EXCEPTION(int, vla.push_back(t));
+                QCOMPARE(vla.size(), 2);
+            }
+        }
+        {
+            const QScopedValueRollback rb(throwOnMoveNow, true);
+            QVERIFY_THROWS_EXCEPTION(int, vla.push_back({}));
+            QCOMPARE(vla.size(), 2);
+        }
     }
 #endif
 }
@@ -919,8 +936,8 @@ void tst_QVarLengthArray::count()
     // tests size(), count() and length(), since they're the same thing
     {
         const QVarLengthArray<int> list;
-        QCOMPARE(list.length(), 0);
-        QCOMPARE(list.count(), 0);
+        QCOMPARE(list.size(), 0);
+        QCOMPARE(list.size(), 0);
         QCOMPARE(list.size(), 0);
         QVERIFY(list.isEmpty());
     }
@@ -928,8 +945,8 @@ void tst_QVarLengthArray::count()
     {
         QVarLengthArray<int> list;
         list.append(0);
-        QCOMPARE(list.length(), 1);
-        QCOMPARE(list.count(), 1);
+        QCOMPARE(list.size(), 1);
+        QCOMPARE(list.size(), 1);
         QCOMPARE(list.size(), 1);
         QVERIFY(!list.isEmpty());
     }
@@ -938,8 +955,8 @@ void tst_QVarLengthArray::count()
         QVarLengthArray<int> list;
         list.append(0);
         list.append(1);
-        QCOMPARE(list.length(), 2);
-        QCOMPARE(list.count(), 2);
+        QCOMPARE(list.size(), 2);
+        QCOMPARE(list.size(), 2);
         QCOMPARE(list.size(), 2);
         QVERIFY(!list.isEmpty());
     }
@@ -949,8 +966,8 @@ void tst_QVarLengthArray::count()
         list.append(0);
         list.append(0);
         list.append(0);
-        QCOMPARE(list.length(), 3);
-        QCOMPARE(list.count(), 3);
+        QCOMPARE(list.size(), 3);
+        QCOMPARE(list.size(), 3);
         QCOMPARE(list.size(), 3);
         QVERIFY(!list.isEmpty());
     }
@@ -961,23 +978,23 @@ void tst_QVarLengthArray::count()
         list.append(0);
         list.append(0);
         list.append(0);
-        QCOMPARE(list.length(), 3);
-        QCOMPARE(list.count(), 3);
+        QCOMPARE(list.size(), 3);
+        QCOMPARE(list.size(), 3);
         QCOMPARE(list.size(), 3);
         QVERIFY(!list.isEmpty());
         list.removeLast();
-        QCOMPARE(list.length(), 2);
-        QCOMPARE(list.count(), 2);
+        QCOMPARE(list.size(), 2);
+        QCOMPARE(list.size(), 2);
         QCOMPARE(list.size(), 2);
         QVERIFY(!list.isEmpty());
         list.removeLast();
-        QCOMPARE(list.length(), 1);
-        QCOMPARE(list.count(), 1);
+        QCOMPARE(list.size(), 1);
+        QCOMPARE(list.size(), 1);
         QCOMPARE(list.size(), 1);
         QVERIFY(!list.isEmpty());
         list.removeLast();
-        QCOMPARE(list.length(), 0);
-        QCOMPARE(list.count(), 0);
+        QCOMPARE(list.size(), 0);
+        QCOMPARE(list.size(), 0);
         QCOMPARE(list.size(), 0);
         QVERIFY(list.isEmpty());
     }
@@ -985,7 +1002,6 @@ void tst_QVarLengthArray::count()
 
 void tst_QVarLengthArray::cpp17ctad()
 {
-#ifdef __cpp_deduction_guides
 #define QVERIFY_IS_VLA_OF(obj, Type) \
     QVERIFY2((std::is_same<decltype(obj), QVarLengthArray<Type>>::value), \
              QMetaType::fromType<decltype(obj)::value_type>().name())
@@ -1005,10 +1021,6 @@ void tst_QVarLengthArray::cpp17ctad()
     CHECK(QString, QStringLiteral("one"), QStringLiteral("two"), QStringLiteral("three"));
 #undef QVERIFY_IS_VLA_OF
 #undef CHECK
-#else
-    QSKIP("This test requires C++17 Constructor Template Argument Deduction support enabled in the compiler.");
-#endif
-
 }
 
 void tst_QVarLengthArray::first()
@@ -1021,16 +1033,16 @@ void tst_QVarLengthArray::first()
     QCOMPARE(list.first(), 27);
     list.append(1987);
     QCOMPARE(list.first(), 27);
-    QCOMPARE(list.length(), 3);
+    QCOMPARE(list.size(), 3);
 
     // remove some, make sure it stays sane
     list.removeLast();
     QCOMPARE(list.first(), 27);
-    QCOMPARE(list.length(), 2);
+    QCOMPARE(list.size(), 2);
 
     list.removeLast();
     QCOMPARE(list.first(), 27);
-    QCOMPARE(list.length(), 1);
+    QCOMPARE(list.size(), 1);
 }
 
 void tst_QVarLengthArray::last()
@@ -1043,16 +1055,16 @@ void tst_QVarLengthArray::last()
     QCOMPARE(list.last(), 4);
     list.append(1987);
     QCOMPARE(list.last(), 1987);
-    QCOMPARE(list.length(), 3);
+    QCOMPARE(list.size(), 3);
 
     // remove some, make sure it stays sane
     list.removeLast();
     QCOMPARE(list.last(), 4);
-    QCOMPARE(list.length(), 2);
+    QCOMPARE(list.size(), 2);
 
     list.removeLast();
     QCOMPARE(list.last(), 27);
-    QCOMPARE(list.length(), 1);
+    QCOMPARE(list.size(), 1);
 }
 
 void tst_QVarLengthArray::squeeze()
@@ -1096,7 +1108,7 @@ void tst_QVarLengthArray::operators()
 
     // +=: not provided, emulate
     //myvla += myvlatwo;
-    for (const QString &s : qAsConst(myvlatwo))
+    for (const QString &s : std::as_const(myvlatwo))
         myvla.push_back(s);
     QCOMPARE(myvla, combined);
 
@@ -1309,7 +1321,7 @@ void tst_QVarLengthArray::insertMove()
         QCOMPARE(MyBase::liveCount, 6);
         QCOMPARE(MyBase::movedCount, 2);
 
-        vec.prepend(std::move(m1));
+        vec.insert(vec.cbegin(), std::move(m1));
         QVERIFY(m1.wasConstructedAt(nullptr));
         QVERIFY(vec.at(0).wasConstructedAt(&m1));
         QVERIFY(vec.at(1).wasConstructedAt(&m3));
@@ -1381,7 +1393,7 @@ void tst_QVarLengthArray::nonCopyable()
     QVERIFY(!val4);
     QVERIFY(ptr3 == vec.at(0).get());
     QVERIFY(ptr4 == vec.at(1).get());
-    vec.prepend(std::move(val1));
+    vec.insert(vec.cbegin(), std::move(val1));
     QVERIFY(!val1);
     QVERIFY(ptr1 == vec.at(0).get());
     QVERIFY(ptr3 == vec.at(1).get());
