@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QtNetwork/qtnetworkglobal.h>
 
@@ -62,6 +37,8 @@ Q_DECLARE_METATYPE(QNetworkRequest::Attribute)
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 QHttp2Configuration qt_defaultH2Configuration()
 {
     QHttp2Configuration config;
@@ -92,6 +69,7 @@ public slots:
     void init();
 private slots:
     // Tests:
+    void defaultQnamHttp2Configuration();
     void singleRequest_data();
     void singleRequest();
     void multipleRequests();
@@ -106,12 +84,21 @@ private slots:
     void maxFrameSize();
     void http2DATAFrames();
 
+    void moreActivitySignals_data();
+    void moreActivitySignals();
+
     void contentEncoding_data();
     void contentEncoding();
 
     void authenticationRequired_data();
     void authenticationRequired();
 
+    void h2cAllowedAttribute_data();
+    void h2cAllowedAttribute();
+
+    void redirect_data();
+    void redirect();
+
 protected slots:
     // Slots to listen to our in-process server:
     void serverStarted(quint16 port);
@@ -220,7 +207,12 @@ tst_Http2::~tst_Http2()
 void tst_Http2::init()
 {
     manager.reset(new QNetworkAccessManager);
-    qputenv("QT_NETWORK_H2C_ALLOWED", "1");
+}
+
+void tst_Http2::defaultQnamHttp2Configuration()
+{
+    // The configuration we also implicitly use in QNAM.
+    QCOMPARE(qt_defaultH2Configuration(), QNetworkRequest().http2Configuration());
 }
 
 void tst_Http2::singleRequest_data()
@@ -274,6 +266,7 @@ void tst_Http2::singleRequest()
     url.setPath("/index.html");
 
     QNetworkRequest request(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
     QFETCH(const QNetworkRequest::Attribute, h2Attribute);
     request.setAttribute(h2Attribute, QVariant(true));
 
@@ -299,7 +292,7 @@ void tst_Http2::singleRequest()
 
 #if QT_CONFIG(ssl)
     if (connectionType == H2Type::h2Alpn || connectionType == H2Type::h2Direct)
-        QCOMPARE(encSpy.count(), 1);
+        QCOMPARE(encSpy.size(), 1);
 #endif // QT_CONFIG(ssl)
 }
 
@@ -449,6 +442,7 @@ void tst_Http2::pushPromise()
     url.setPath("/index.html");
 
     QNetworkRequest request(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
     request.setAttribute(QNetworkRequest::Http2AllowedAttribute, QVariant(true));
     request.setHttp2Configuration(params);
 
@@ -475,6 +469,7 @@ void tst_Http2::pushPromise()
 
     url.setPath("/script.js");
     QNetworkRequest promisedRequest(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
     promisedRequest.setAttribute(QNetworkRequest::Http2AllowedAttribute, QVariant(true));
     reply = manager->get(promisedRequest);
     connect(reply, &QNetworkReply::finished, this, &tst_Http2::replyFinished);
@@ -529,6 +524,7 @@ void tst_Http2::goaway()
     for (int i = 0; i < nRequests; ++i) {
         url.setPath(QString("/%1").arg(i));
         QNetworkRequest request(url);
+        request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
         request.setAttribute(QNetworkRequest::Http2AllowedAttribute, QVariant(true));
         replies[i] = manager->get(request);
         QCOMPARE(replies[i]->error(), QNetworkReply::NoError);
@@ -670,6 +666,7 @@ void tst_Http2::connectToHost()
         auto copyUrl = url;
         copyUrl.setScheme(QLatin1String("preconnect-https"));
         QNetworkRequest request(copyUrl);
+        request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
         request.setAttribute(requestAttribute, true);
         reply = manager->get(request);
         // Since we're using self-signed certificates, ignore SSL errors:
@@ -682,6 +679,7 @@ void tst_Http2::connectToHost()
         auto copyUrl = url;
         copyUrl.setScheme(QLatin1String("preconnect-http"));
         QNetworkRequest request(copyUrl);
+        request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
         request.setAttribute(requestAttribute, true);
         reply = manager->get(request);
     }
@@ -702,6 +700,7 @@ void tst_Http2::connectToHost()
     QCOMPARE(nRequests, 1);
 
     QNetworkRequest request(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
     request.setAttribute(requestAttribute, QVariant(true));
     reply = manager->get(request);
     connect(reply, &QNetworkReply::finished, this, &tst_Http2::replyFinished);
@@ -767,6 +766,7 @@ void tst_Http2::maxFrameSize()
     url.setPath(QString("/stream1.html"));
 
     QNetworkRequest request(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
     request.setAttribute(attribute, QVariant(true));
     request.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("text/plain"));
     request.setHttp2Configuration(h2Config);
@@ -780,7 +780,7 @@ void tst_Http2::maxFrameSize()
 
     // Normally, with a 16kb limit, our server would split such
     // a response into 3 'DATA' frames (16kb + 16kb + 0|END_STREAM).
-    QCOMPARE(frameCounter.count(), 1);
+    QCOMPARE(frameCounter.size(), 1);
 
     QVERIFY(nRequests == 0);
     QVERIFY(prefaceOK);
@@ -870,6 +870,78 @@ void tst_Http2::http2DATAFrames()
     }
 }
 
+void tst_Http2::moreActivitySignals_data()
+{
+    QTest::addColumn<QNetworkRequest::Attribute>("h2Attribute");
+    QTest::addColumn<H2Type>("connectionType");
+
+    QTest::addRow("h2c-upgrade")
+            << QNetworkRequest::Http2AllowedAttribute << H2Type::h2c;
+    QTest::addRow("h2c-direct")
+            << QNetworkRequest::Http2DirectAttribute << H2Type::h2cDirect;
+
+    if (!clearTextHTTP2)
+        QTest::addRow("h2-ALPN")
+                << QNetworkRequest::Http2AllowedAttribute << H2Type::h2Alpn;
+
+#if QT_CONFIG(ssl)
+    QTest::addRow("h2-direct")
+            << QNetworkRequest::Http2DirectAttribute << H2Type::h2Direct;
+#endif
+}
+
+void tst_Http2::moreActivitySignals()
+{
+    clearHTTP2State();
+
+#if QT_CONFIG(securetransport)
+    // Normally on macOS we use plain text only for SecureTransport
+    // does not support ALPN on the server side. With 'direct encrytped'
+    // we have to use TLS sockets (== private key) and thus suppress a
+    // keychain UI asking for permission to use a private key.
+    // Our CI has this, but somebody testing locally - will have a problem.
+    qputenv("QT_SSL_USE_TEMPORARY_KEYCHAIN", QByteArray("1"));
+    auto envRollback = qScopeGuard([]() { qunsetenv("QT_SSL_USE_TEMPORARY_KEYCHAIN"); });
+#endif
+
+    serverPort = 0;
+    QFETCH(H2Type, connectionType);
+    ServerPtr srv(newServer(defaultServerSettings, connectionType));
+    QMetaObject::invokeMethod(srv.data(), "startServer", Qt::QueuedConnection);
+    runEventLoop(100);
+    QVERIFY(serverPort != 0);
+    auto url = requestUrl(connectionType);
+    url.setPath(QString("/stream1.html"));
+    QNetworkRequest request(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
+    QFETCH(const QNetworkRequest::Attribute, h2Attribute);
+    request.setAttribute(h2Attribute, QVariant(true));
+    request.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("text/plain"));
+    QSharedPointer<QNetworkReply> reply(manager->get(request));
+    nRequests = 1;
+    connect(reply.data(), &QNetworkReply::finished, this, &tst_Http2::replyFinished);
+    QSignalSpy spy1(reply.data(), SIGNAL(socketStartedConnecting()));
+    QSignalSpy spy2(reply.data(), SIGNAL(requestSent()));
+    QSignalSpy spy3(reply.data(), SIGNAL(metaDataChanged()));
+    // Since we're using self-signed certificates,
+    // ignore SSL errors:
+    reply->ignoreSslErrors();
+
+    spy1.wait();
+    spy2.wait();
+    spy3.wait();
+
+    runEventLoop();
+    STOP_ON_FAILURE
+
+    QVERIFY(nRequests == 0);
+    QVERIFY(prefaceOK);
+    QVERIFY(serverGotSettingsACK);
+
+    QVERIFY(reply->error() == QNetworkReply::NoError);
+    QVERIFY(reply->isFinished());
+}
+
 void tst_Http2::contentEncoding_data()
 {
     QTest::addColumn<QByteArray>("encoding");
@@ -962,6 +1034,7 @@ void tst_Http2::contentEncoding()
     url.setPath("/index.html");
 
     QNetworkRequest request(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
     QFETCH(const QNetworkRequest::Attribute, h2Attribute);
     request.setAttribute(h2Attribute, QVariant(true));
 
@@ -1019,6 +1092,7 @@ void tst_Http2::authenticationRequired()
     auto url = requestUrl(defaultConnectionType());
     url.setPath("/index.html");
     QNetworkRequest request(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
 
     QByteArray expectedBody = "Hello, World!";
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
@@ -1073,6 +1147,139 @@ void tst_Http2::authenticationRequired()
     QTRY_VERIFY(serverGotSettingsACK);
 }
 
+void tst_Http2::h2cAllowedAttribute_data()
+{
+    QTest::addColumn<bool>("h2cAllowed");
+    QTest::addColumn<bool>("useAttribute"); // true: use attribute, false: use environment variable
+    QTest::addColumn<bool>("success");
+
+    QTest::addRow("h2c-not-allowed") << false << false << false;
+    // Use the attribute to enable/disable the H2C:
+    QTest::addRow("attribute") << true << true << true;
+    // Use the QT_NETWORK_H2C_ALLOWED environment variable to enable/disable the H2C:
+    QTest::addRow("environment-variable") << true << false << true;
+}
+
+void tst_Http2::h2cAllowedAttribute()
+{
+    QFETCH(const bool, h2cAllowed);
+    QFETCH(const bool, useAttribute);
+    QFETCH(const bool, success);
+
+    clearHTTP2State();
+    serverPort = 0;
+
+    ServerPtr targetServer(newServer(defaultServerSettings, H2Type::h2c));
+    targetServer->setResponseBody("Hello");
+
+    QMetaObject::invokeMethod(targetServer.data(), "startServer", Qt::QueuedConnection);
+    runEventLoop();
+
+    QVERIFY(serverPort != 0);
+
+    nRequests = 1;
+
+    auto url = requestUrl(H2Type::h2c);
+    url.setPath("/index.html");
+    QNetworkRequest request(url);
+    if (h2cAllowed) {
+        if (useAttribute)
+            request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
+        else
+            qputenv("QT_NETWORK_H2C_ALLOWED", "1");
+    }
+    auto envCleanup = qScopeGuard([]() { qunsetenv("QT_NETWORK_H2C_ALLOWED"); });
+
+    QScopedPointer<QNetworkReply> reply;
+    reply.reset(manager->get(request));
+
+    if (success)
+        connect(reply.get(), &QNetworkReply::finished, this, &tst_Http2::replyFinished);
+    else
+        connect(reply.get(), &QNetworkReply::errorOccurred, this, &tst_Http2::replyFinishedWithError);
+
+    // Since we're using self-signed certificates,
+    // ignore SSL errors:
+    reply->ignoreSslErrors();
+
+    runEventLoop();
+    STOP_ON_FAILURE
+
+    if (!success) {
+        QCOMPARE(reply->error(), QNetworkReply::ConnectionRefusedError);
+    } else {
+        QCOMPARE(reply->readAll(), QByteArray("Hello"));
+        QTRY_VERIFY(serverGotSettingsACK);
+    }
+}
+
+void tst_Http2::redirect_data()
+{
+    QTest::addColumn<int>("maxRedirects");
+    QTest::addColumn<int>("redirectCount");
+    QTest::addColumn<bool>("success");
+
+    QTest::addRow("1-redirects-none-allowed-failure") << 0 << 1 << false;
+    QTest::addRow("1-redirects-success") << 1 << 1 << true;
+    QTest::addRow("2-redirects-1-allowed-failure") << 1 << 2 << false;
+}
+
+void tst_Http2::redirect()
+{
+    QFETCH(const int, maxRedirects);
+    QFETCH(const int, redirectCount);
+    QFETCH(const bool, success);
+    const QByteArray redirectUrl = "/b.html"_ba;
+
+    clearHTTP2State();
+    serverPort = 0;
+
+    ServerPtr targetServer(newServer(defaultServerSettings, defaultConnectionType()));
+    targetServer->setRedirect(redirectUrl, redirectCount);
+
+    QMetaObject::invokeMethod(targetServer.data(), "startServer", Qt::QueuedConnection);
+    runEventLoop();
+
+    QVERIFY(serverPort != 0);
+
+    nRequests = 1 + maxRedirects;
+
+    auto originalUrl = requestUrl(defaultConnectionType());
+    auto url = originalUrl;
+    url.setPath("/index.html");
+    QNetworkRequest request(url);
+    request.setMaximumRedirectsAllowed(maxRedirects);
+    // H2C might be used on macOS where SecureTransport doesn't support server-side ALPN
+    qputenv("QT_NETWORK_H2C_ALLOWED", "1");
+    auto envCleanup = qScopeGuard([]() { qunsetenv("QT_NETWORK_H2C_ALLOWED"); });
+
+    QScopedPointer<QNetworkReply> reply;
+    reply.reset(manager->get(request));
+
+    if (success) {
+        connect(reply.get(), &QNetworkReply::finished, this, &tst_Http2::replyFinished);
+    } else {
+        connect(reply.get(), &QNetworkReply::errorOccurred, this,
+                &tst_Http2::replyFinishedWithError);
+    }
+
+    // Since we're using self-signed certificates,
+    // ignore SSL errors:
+    reply->ignoreSslErrors();
+
+    runEventLoop();
+    STOP_ON_FAILURE
+
+    if (success) {
+        QCOMPARE(reply->error(), QNetworkReply::NoError);
+        QCOMPARE(reply->url().toString(),
+                 originalUrl.resolved(QString::fromLatin1(redirectUrl)).toString());
+    } else if (maxRedirects < redirectCount) {
+        QCOMPARE(reply->error(), QNetworkReply::TooManyRedirectsError);
+    }
+    QTRY_VERIFY(serverGotSettingsACK);
+}
+
 void tst_Http2::serverStarted(quint16 port)
 {
     serverPort = port;
@@ -1130,6 +1337,7 @@ void tst_Http2::sendRequest(int streamNumber,
     url.setPath(QString("/stream%1.html").arg(streamNumber));
 
     QNetworkRequest request(url);
+    request.setAttribute(QNetworkRequest::Http2CleartextAllowedAttribute, true);
     request.setAttribute(QNetworkRequest::Http2AllowedAttribute, QVariant(true));
     request.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
     request.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("text/plain"));
