@@ -1,36 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 
 #include <QTest>
 #include <QtTest/qtesttouch.h>
 
 #include <qevent.h>
+#include <QSet>
 #include <qpointingdevice.h>
 #include <qwidget.h>
 #include <qlayout.h>
@@ -342,6 +318,8 @@ private slots:
     void testQGestureRecognizerCleanup();
     void testReuseCanceledGestures();
     void bug_13501_gesture_not_accepted();
+private:
+    QPoint m_availableTopLeft;
 };
 
 void tst_Gestures::initTestCase()
@@ -349,6 +327,8 @@ void tst_Gestures::initTestCase()
     CustomGesture::GestureType = QGestureRecognizer::registerRecognizer(new CustomGestureRecognizer);
     QVERIFY(CustomGesture::GestureType != Qt::GestureType(0));
     QVERIFY(CustomGesture::GestureType != Qt::CustomGesture);
+    const QScreen *screen = QGuiApplication::primaryScreen();
+    m_availableTopLeft = screen->availableGeometry().topLeft();
 }
 
 void tst_Gestures::cleanupTestCase()
@@ -604,7 +584,7 @@ void tst_Gestures::conflictingGestures()
 
     QCOMPARE(child->gestureOverrideEventsReceived, 1);
     QVERIFY(child->gestureEventsReceived > TotalGestureEventsCount);
-    QCOMPARE(child->events.all.count(), TotalGestureEventsCount + ContinuousGestureEventsCount);
+    QCOMPARE(child->events.all.size(), TotalGestureEventsCount + ContinuousGestureEventsCount);
     QCOMPARE(parent.gestureOverrideEventsReceived, 0);
     QCOMPARE(parent.gestureEventsReceived, 0);
 
@@ -824,6 +804,7 @@ void tst_Gestures::graphicsItemGesture()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     GestureItem *item = new GestureItem("item");
     scene.addItem(item);
@@ -886,6 +867,7 @@ void tst_Gestures::graphicsView()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     GestureItem *item = new GestureItem("item");
     scene.addItem(item);
@@ -951,6 +933,7 @@ void tst_Gestures::graphicsItemTreeGesture()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     GestureItem *item1 = new GestureItem("item1");
     item1->setPos(100, 100);
@@ -1008,6 +991,7 @@ void tst_Gestures::explicitGraphicsObjectTarget()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     GestureItem *item1 = new GestureItem("item1");
     scene.addItem(item1);
@@ -1425,13 +1409,13 @@ void tst_Gestures::ungrabGesture() // a method on QWidget
     // sending an event will cause the QGesture objects to be instantiated for the widgets
     sendCustomGesture(&event, b);
 
-    QCOMPARE(a->gestures.count(), 1);
+    QCOMPARE(a->gestures.size(), 1);
     QPointer<QGesture> customGestureA;
     customGestureA = *(a->gestures.begin());
     QVERIFY(!customGestureA.isNull());
     QCOMPARE(customGestureA->gestureType(), CustomGesture::GestureType);
 
-    QCOMPARE(b->gestures.count(), 1);
+    QCOMPARE(b->gestures.size(), 1);
     QPointer<QGesture> customGestureB;
     customGestureB = *(b->gestures.begin());
     QVERIFY(!customGestureB.isNull());
@@ -1442,7 +1426,7 @@ void tst_Gestures::ungrabGesture() // a method on QWidget
     // sending an event will cause the QGesture objects to be instantiated for the widget
     sendCustomGesture(&event, a);
 
-    QCOMPARE(a->gestures.count(), 1);
+    QCOMPARE(a->gestures.size(), 1);
     customGestureA = *(a->gestures.begin());
     QVERIFY(!customGestureA.isNull());
     QCOMPARE(customGestureA->gestureType(), CustomGesture::GestureType);
@@ -1487,7 +1471,7 @@ void tst_Gestures::autoCancelGestures()
         {
             if (event->type() == QEvent::Gesture) {
                 QGestureEvent *ge = static_cast<QGestureEvent*>(event);
-                if (ge->gestures().count() != 1)
+                if (ge->gestures().size() != 1)
                     ++badGestureEvents;   // event should contain exactly one gesture
                 ge->gestures().first()->setGestureCancelPolicy(QGesture::CancelAllInContext);
             }
@@ -1520,15 +1504,15 @@ void tst_Gestures::autoCancelGestures()
     CustomEvent event;
     event.serial = CustomGesture::SerialStartedThreshold;
     QApplication::sendEvent(child, &event);
-    QCOMPARE(child->events.all.count(), 2);
-    QCOMPARE(child->events.started.count(), 1);
-    QCOMPARE(child->events.canceled.count(), 1);
-    QCOMPARE(parent.events.all.count(), 1);
+    QCOMPARE(child->events.all.size(), 2);
+    QCOMPARE(child->events.started.size(), 1);
+    QCOMPARE(child->events.canceled.size(), 1);
+    QCOMPARE(parent.events.all.size(), 1);
 
     // clean up, make the parent gesture finish
     event.serial = CustomGesture::SerialFinishedThreshold;
     QApplication::sendEvent(child, &event);
-    QCOMPARE(parent.events.all.count(), 2);
+    QCOMPARE(parent.events.all.size(), 2);
     QCOMPARE(parent.badGestureEvents, 0);
 }
 
@@ -1542,7 +1526,7 @@ void tst_Gestures::autoCancelGestures2()
         {
             if (event->type() == QEvent::Gesture) {
                 QGestureEvent *ge = static_cast<QGestureEvent*>(event);
-                if (ge->gestures().count() != 1)
+                if (ge->gestures().size() != 1)
                     ++badGestureEvents;   // event should contain exactly one gesture
                 ge->gestures().first()->setGestureCancelPolicy(QGesture::CancelAllInContext);
             }
@@ -1557,6 +1541,7 @@ void tst_Gestures::autoCancelGestures2()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     MockItem *parent = new MockItem("parent");
     GestureItem *child = new GestureItem("child");
@@ -1576,15 +1561,15 @@ void tst_Gestures::autoCancelGestures2()
     event.hasHotSpot = true;
     event.hotSpot = mapToGlobal(QPointF(5, 5), child, &view);
     scene.sendEvent(child, &event);
-    QCOMPARE(parent->events.all.count(), 1);
-    QCOMPARE(child->events.started.count(), 1);
-    QCOMPARE(child->events.canceled.count(), 1);
-    QCOMPARE(child->events.all.count(), 2);
+    QCOMPARE(parent->events.all.size(), 1);
+    QCOMPARE(child->events.started.size(), 1);
+    QCOMPARE(child->events.canceled.size(), 1);
+    QCOMPARE(child->events.all.size(), 2);
 
     // clean up, make the parent gesture finish
     event.serial = CustomGesture::SerialFinishedThreshold;
     scene.sendEvent(child, &event);
-    QCOMPARE(parent->events.all.count(), 2);
+    QCOMPARE(parent->events.all.size(), 2);
     QCOMPARE(parent->badGestureEvents, 0);
 }
 
@@ -1593,6 +1578,7 @@ void tst_Gestures::graphicsViewParentPropagation()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     GestureItem *item0 = new GestureItem("item0");
     scene.addItem(item0);
@@ -1653,6 +1639,7 @@ void tst_Gestures::panelPropagation()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     GestureItem *item0 = new GestureItem("item0");
     scene.addItem(item0);
@@ -1777,6 +1764,7 @@ void tst_Gestures::panelStacksBehindParent()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     GestureItem *item1 = new GestureItem("item1");
     item1->grabGesture(CustomGesture::GestureType);
@@ -1964,6 +1952,7 @@ void tst_Gestures::partialGesturePropagation()
     QGraphicsScene scene;
     QGraphicsView view(&scene);
     view.setWindowFlags(Qt::X11BypassWindowManagerHint);
+    view.move(m_availableTopLeft);
 
     GestureItem *item1 = new GestureItem("item1");
     item1->grabGesture(CustomGesture::GestureType);
@@ -2176,6 +2165,7 @@ void tst_Gestures::testReuseCanceledGestures()
     mw.setWindowFlags(Qt::X11BypassWindowManagerHint);
     QGraphicsView *gv = new QGraphicsView(&mw);
     QGraphicsScene *scene = new QGraphicsScene;
+    mw.move(m_availableTopLeft);
 
     gv->setScene(scene);
     scene->setSceneRect(0,0,100,100);
