@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 
 #include <QTest>
@@ -695,7 +670,7 @@ public:
 
 private slots:
     void valueChanged(qreal value) {
-        for (int i = 0; i < fromGeoms.count(); ++i) {
+        for (int i = 0; i < fromGeoms.size(); ++i) {
             QGraphicsLayoutItem *li = itemAt(i);
             QRectF from = fromGeoms.at(i);
             QRectF to = toGeoms.at(i);
@@ -850,7 +825,7 @@ CustomLayout(QGraphicsLayoutItem *parent)
 
 int count() const override
 {
-    return items.count();
+    return items.size();
 }
 
 QGraphicsLayoutItem* itemAt(int index) const override
@@ -866,12 +841,12 @@ void removeAt(int index) override
 
 void addItem(QGraphicsLayoutItem *item)
 {
-    insertItem(items.count(), item);
+    insertItem(items.size(), item);
 }
 
 void insertItem(int index, QGraphicsLayoutItem *item)
 {
-    index = qBound(0, index, items.count());
+    index = qBound(0, index, items.size());
 
     item->setParentLayoutItem(this);
 
@@ -879,7 +854,7 @@ void insertItem(int index, QGraphicsLayoutItem *item)
     updateParentWidget(widget);
 
 
-    if (index == items.count()) {
+    if (index == items.size()) {
         items.append(item);
     } else {
         items.insert(index, item);
@@ -947,7 +922,7 @@ void tst_QGraphicsLayout::ownership()
 
         destructedSet.clear();
         window->setLayout(0);
-        QCOMPARE(destructedSet.count(), 0);
+        QCOMPARE(destructedSet.size(), 0);
         delete window;
     }
 
