@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the FOO module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #ifndef BENCHMARKTESTS_H
 #define BENCHMARKTESTS_H
@@ -554,13 +529,13 @@ public:
             m_staticTexts.append(staticText);
 
             QFontMetrics fm(p->font());
-            m_size = QSize(fm.horizontalAdvance(m_text, m_text.length()), fm.height());
+            m_size = QSize(fm.horizontalAdvance(m_text, m_text.size()), fm.height());
 
             break;
         }
         case PainterQPointMode: {
             QFontMetrics fm(p->font());
-            m_size = QSize(fm.horizontalAdvance(m_text, m_text.length()), fm.height());
+            m_size = QSize(fm.horizontalAdvance(m_text, m_text.size()), fm.height());
             break;
         }
 
@@ -601,7 +576,7 @@ public:
 
     QString name() const override
     {
-        int letters = m_text.length();
+        int letters = m_text.size();
         int lines = m_text.count('\n');
         if (lines == 0)
             lines = 1;
