@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 // This file contains benchmarks for QNetworkReply functions.
 
 #include <QDebug>
@@ -631,9 +606,11 @@ void tst_qnetworkreply::uploadPerformance()
       QVERIFY(!QTestEventLoop::instance().timeout());
 }
 
+constexpr qint64 MiB = 1024 * 1024;
+
 void tst_qnetworkreply::httpUploadPerformance()
 {
-      enum {UploadSize = 128*1024*1024}; // 128 MB
+      constexpr qint64 UploadSize = 128 * MiB;
 
       ThreadedDataReaderHttpServer reader;
       FixedSizeDataGenerator generator(UploadSize);
@@ -701,7 +678,7 @@ void tst_qnetworkreply::httpDownloadPerformance()
     QFETCH(bool, serverSendsContentLength);
     QFETCH(bool, chunkedEncoding);
 
-    enum {UploadSize = 128*1024*1024}; // 128 MB
+    constexpr qint64 UploadSize = 128 * MiB;
 
     HttpDownloadPerformanceServer server(UploadSize, serverSendsContentLength, chunkedEncoding);
 
